@@ -30,6 +30,9 @@
 #include <memory>
 #include <set>
 
+#include <fmt/format.h>
+#include <fmt/ostream.h>
+
 #include <log4cxx/basicconfigurator.h>
 #include <log4cxx/logger.h>
 #include <log4cxx/rollingfileappender.h>
@@ -316,9 +319,9 @@ void Application::AutoMaintenance()
   // must build language.dat if:
   //   (1) in user mode and an admin just modified the MiKTeX configuration
   //   (2) in user mode and languages.ini is newer than languages.dat
-  PathName userLanguageDat = pimpl->session->IsAdminMode() ? "" : pimpl->session->GetSpecialPath(SpecialPath::UserConfigRoot) / MIKTEX_PATH_LANGUAGE_DAT;
+  PathName userLanguageDat = pimpl->session->IsAdminMode() ? PathName() : pimpl->session->GetSpecialPath(SpecialPath::UserConfigRoot) / PathName(MIKTEX_PATH_LANGUAGE_DAT);
   bool mustRefreshUserLanguageDat = !pimpl->session->IsAdminMode() && File::Exists(userLanguageDat) && lastAdminMaintenance > File::GetLastWriteTime(userLanguageDat);
-  PathName userLanguagesIni = pimpl->session->IsAdminMode() ? "" : pimpl->session->GetSpecialPath(SpecialPath::UserConfigRoot) / MIKTEX_PATH_LANGUAGES_INI;
+  PathName userLanguagesIni = pimpl->session->IsAdminMode() ? PathName() : pimpl->session->GetSpecialPath(SpecialPath::UserConfigRoot) / PathName(MIKTEX_PATH_LANGUAGES_INI);
   mustRefreshUserLanguageDat = mustRefreshUserLanguageDat || (!pimpl->session->IsAdminMode() && IsNewer(userLanguagesIni, userLanguageDat));
 
   // must update package db if:
@@ -327,14 +330,14 @@ void Application::AutoMaintenance()
   if (!pimpl->session->IsAdminMode())
   {
     time_t lastAdminUpdateDb = pimpl->session->GetConfigValue(MIKTEX_CONFIG_SECTION_MPM, MIKTEX_CONFIG_VALUE_LAST_ADMIN_UPDATE_DB, ConfigValue("0")).GetTimeT();
-    PathName userPackageManifestsIni = pimpl->session->GetSpecialPath(SpecialPath::InstallRoot) / MIKTEX_PATH_PACKAGE_MANIFESTS_INI;
+    PathName userPackageManifestsIni = pimpl->session->GetSpecialPath(SpecialPath::InstallRoot) / PathName(MIKTEX_PATH_PACKAGE_MANIFESTS_INI);
     mustUpdateDb = File::Exists(userPackageManifestsIni) && lastAdminUpdateDb > File::GetLastWriteTime(userPackageManifestsIni);
   }
 
   PathName initexmf;
   if ((mustRefreshFndb || mustRefreshUserLanguageDat || mustUpdateDb) && pimpl->session->FindFile(MIKTEX_INITEXMF_EXE, FileType::EXE, initexmf))
   {
-    unique_ptr<MiKTeX::Core::LockFile> lockFile = LockFile::Create(pimpl->session->GetSpecialPath(SpecialPath::DataRoot) / MIKTEX_PATH_AUTO_MAINTENANCE_LOCK);
+    unique_ptr<MiKTeX::Core::LockFile> lockFile = LockFile::Create(pimpl->session->GetSpecialPath(SpecialPath::DataRoot) / PathName(MIKTEX_PATH_AUTO_MAINTENANCE_LOCK));
     if (!lockFile->TryLock(0ms))
     {
       return;
@@ -415,7 +418,7 @@ constexpr time_t ONE_WEEK = 7 * ONE_DAY;
 void Application::AutoDiagnose()
 {
   time_t now = time(nullptr);
-  PathName issuesJson = pimpl->session->GetSpecialPath(SpecialPath::ConfigRoot) / MIKTEX_PATH_ISSUES_JSON;
+  PathName issuesJson = pimpl->session->GetSpecialPath(SpecialPath::ConfigRoot) / PathName(MIKTEX_PATH_ISSUES_JSON);
   vector<Setup::Issue> issues;
   if (!File::Exists(issuesJson) || now > File::GetLastWriteTime(issuesJson) + ONE_WEEK)
   {
@@ -799,7 +802,7 @@ TriState Application::GetEnableInstaller() const
   return pimpl->enableInstaller;
 }
 
-void Application::Trace(const TraceCallback::TraceMessage& traceMessage)
+bool Application::Trace(const TraceCallback::TraceMessage& traceMessage)
 {
   if (!isLog4cxxConfigured)
   {
@@ -808,10 +811,11 @@ void Application::Trace(const TraceCallback::TraceMessage& traceMessage)
       pimpl->pendingTraceMessages.clear();
     }
     pimpl->pendingTraceMessages.push_back(traceMessage);
-    return;
+    return true;
   }
   FlushPendingTraceMessages();
   TraceInternal(traceMessage);
+  return true;
 }
 
 void Application::FlushPendingTraceMessages()
@@ -866,12 +870,12 @@ void Application::Sorry(const string& name, const string& description, const str
   cerr << endl;
   if (description.empty())
   {
-    cerr << StringUtil::FormatString(T_("Sorry, but %s did not succeed."), Q_(name)) << endl;
+    cerr << fmt::format(T_("Sorry, but {0} did not succeed."), Q_(name)) << endl;
   }
   else
   {
     cerr
-      << StringUtil::FormatString(T_("Sorry, but %s did not succeed for the following reason:"), Q_(name)) << "\n"
+      << fmt::format(T_("Sorry, but {0} did not succeed for the following reason:"), Q_(name)) << "\n"
       << "\n"
       << "  " << description << endl;
     if (!remedy.empty())
@@ -984,7 +988,7 @@ void Application::InvokeEditor(const PathName& editFileName, int editLineNumber,
   {
     // read information from yap.ini
     // FIXME: use FindFile()
-    PathName yapIni = pimpl->session->GetSpecialPath(SpecialPath::UserConfigRoot) / MIKTEX_PATH_MIKTEX_CONFIG_DIR / MIKTEX_YAP_INI_FILENAME;
+    PathName yapIni = pimpl->session->GetSpecialPath(SpecialPath::UserConfigRoot) / PathName(MIKTEX_PATH_MIKTEX_CONFIG_DIR) / PathName(MIKTEX_YAP_INI_FILENAME);
     if (File::Exists(yapIni))
     {
       unique_ptr<Cfg> yapConfig(Cfg::Create());
@@ -1070,7 +1074,7 @@ void Application::InvokeEditor(const PathName& editFileName, int editLineNumber,
     }
   }
 
-  Process::Start(fileName, Argv(commandLine).ToStringVector());
+  Process::Start(PathName(fileName), Argv(commandLine).ToStringVector());
 }
 
 bool Application::GetQuietFlag() const
