@@ -65,19 +65,35 @@ void winProcess::Create()
 
   PathName fileName;
 
-  if (Utils::IsAbsolutePath(startinfo.FileName))
+  if (PathNameUtil::IsAbsolutePath(startinfo.FileName))
   {
     fileName = startinfo.FileName;
   }
   else
   {
-    wchar_t* lpszFilePart = nullptr;
-    wchar_t szFileName[_MAX_PATH];
-    if (SearchPathW(nullptr, UW_(startinfo.FileName), L".exe", _MAX_PATH, szFileName, &lpszFilePart) == 0)
+    CharBuffer<wchar_t, BufferSizes::MaxPath> foundFileName;
+    bool done = false;
+    int rounds = 0;
+    do
     {
-      MIKTEX_FATAL_WINDOWS_ERROR_2("SearchPath", "fileName", startinfo.FileName);
-    }
-    fileName = szFileName;
+      wchar_t* lpszFilePart = nullptr;
+      DWORD n = SearchPathW(nullptr, UW_(startinfo.FileName), L".exe", foundFileName.GetCapacity(), foundFileName.GetData(), &lpszFilePart);
+      if (n == 0)
+      {
+        MIKTEX_FATAL_WINDOWS_ERROR_2("SearchPath", "fileName", startinfo.FileName);
+      }
+      done = n < foundFileName.GetCapacity();
+      if (!done)
+      {
+        if (rounds > 0)
+        {
+          BUF_TOO_SMALL();
+        }
+        foundFileName.Reserve(n);
+      }
+      rounds++;
+    } while (!done);
+    fileName = foundFileName.GetData();
   }
 
   CommandLineBuilder commandLine;
@@ -110,7 +126,7 @@ void winProcess::Create()
 #if TRACEREDIR
       if (session != nullptr)
       {
-        session->trace_process->WriteFormattedLine("core", "redirecting stdout to a stream");
+        session->trace_process->WriteLine("core", "redirecting stdout to a stream");
       }
 #endif
       int fd = _fileno(startinfo.StandardOutput);
@@ -133,7 +149,7 @@ void winProcess::Create()
 #if TRACEREDIR
       if (session != nullptr)
       {
-        session->trace_process->WriteFormattedLine("core", "redirecting stdout to a pipe");
+        session->trace_process->WriteLine("core", "redirecting stdout to a pipe");
       }
 #endif
       // create stdout pipe
@@ -155,7 +171,7 @@ void winProcess::Create()
 #if TRACEREDIR
       if (session != nullptr)
       {
-        session->trace_process->WriteFormattedLine("core", "redirecting stderr to a stream");
+        session->trace_process->WriteLine("core", "redirecting stderr to a stream");
       }
 #endif
       int fd = _fileno(startinfo.StandardError);
@@ -178,7 +194,7 @@ void winProcess::Create()
 #if TRACEREDIR
       if (session != nullptr)
       {
-        session->trace_process->WriteFormattedLine("core", "redirecting stderr to a pipe");
+        session->trace_process->WriteLine("core", "redirecting stderr to a pipe");
       }
 #endif
       // create child stderr pipe
@@ -199,7 +215,7 @@ void winProcess::Create()
 #if TRACEREDIR
       if (session != nullptr)
       {
-        session->trace_process->WriteFormattedLine("core", "make child stderr = child stdout");
+        session->trace_process->WriteLine("core", "make child stderr = child stdout");
       }
 #endif
       // make child stderr = child stdout
@@ -216,7 +232,7 @@ void winProcess::Create()
 #if TRACEREDIR
       if (session != nullptr)
       {
-        session->trace_process->WriteFormattedLine("core", "redirecting stdin to a stream");
+        session->trace_process->WriteLine("core", "redirecting stdin to a stream");
       }
 #endif
       int fd = _fileno(startinfo.StandardInput);
@@ -239,7 +255,7 @@ void winProcess::Create()
 #if TRACEREDIR
       if (session != nullptr)
       {
-        session->trace_process->WriteFormattedLine("core", "redirecting stdin to a pipe");
+        session->trace_process->WriteLine("core", "redirecting stdin to a pipe");
       }
 #endif
       // create child stdin pipe
@@ -495,7 +511,7 @@ MIKTEXSTATICFUNC(PathName) FindSystemShell()
     string path;
     if (Utils::GetEnvironmentString("COMSPEC", path))
     {
-      if (!Utils::IsAbsolutePath(path))
+      if (!PathNameUtil::IsAbsolutePath(path))
       {
         wchar_t* lpsz = nullptr;
         if (SearchPathW(nullptr, PathName(path).ToWideCharString().c_str(), nullptr, ARRAY_SIZE(szCmd), szCmd, &lpsz) == 0)
@@ -503,7 +519,7 @@ MIKTEXSTATICFUNC(PathName) FindSystemShell()
           szCmd[0] = 0;
         }
       }
-      else if (File::Exists(path))
+      else if (File::Exists(PathName(path)))
       {
         StringUtil::CopyString(szCmd, ARRAY_SIZE(szCmd), PathName(path).ToWideCharString().c_str());
       }
@@ -516,7 +532,7 @@ MIKTEXSTATICFUNC(PathName) FindSystemShell()
     }
   }
 
-  return szCmd;
+  return PathName(szCmd);
 }
 
 MIKTEXSTATICFUNC(vector<string>) Wrap(const string& commandLine)
@@ -531,13 +547,13 @@ MIKTEXSTATICFUNC(vector<string>) Wrap(const string& commandLine)
 bool Process::ExecuteSystemCommand(const string& commandLine, int* exitCode, IRunProcessCallback* callback, const char* workingDirectory)
 {
   vector<string> arguments = Wrap(commandLine);
-  return Process::Run(arguments[0], arguments, callback, exitCode, workingDirectory);
+  return Process::Run(PathName(arguments[0]), arguments, callback, exitCode, workingDirectory);
 }
 
 void Process::StartSystemCommand(const string& commandLine)
 {
   vector<string> arguments = Wrap(commandLine);
-  Process::Start(arguments[0], arguments);
+  Process::Start(PathName(arguments[0]), arguments);
 }
 
 winProcess::winProcess()
@@ -681,3 +697,10 @@ ProcessInfo winProcess::GetProcessInfo()
   processInfo.status = ProcessStatus::Runnable;
   return processInfo;
 }
+
+void Process::Overlay(const PathName& fileName, const std::vector<std::string>& arguments)
+{
+  int exitCode;
+  Process::Run(fileName, arguments, nullptr, &exitCode, nullptr);
+  throw exitCode;
+}
