@@ -107,7 +107,7 @@ PathName SessionImpl::GetMyPrefix(bool canonicalized)
   })
   {
     PathName prefix;
-    if (Utils::GetPathNamePrefix(bindir, subdir, prefix))
+    if (Utils::GetPathNamePrefix(bindir, PathName(subdir), prefix))
     {
       return prefix;
     }
@@ -154,7 +154,7 @@ bool SessionImpl::FindStartupConfigFile(ConfigurationScope scope, PathName& path
     PathName prefix;
     if (Utils::GetPathNamePrefix(myloc, internalBindir, prefix))
     {
-      path = prefix / MIKTEX_PATH_STARTUP_CONFIG_FILE;
+      path = prefix / PathName(MIKTEX_PATH_STARTUP_CONFIG_FILE);
       if (File::Exists(path))
       {
         return true;
@@ -165,14 +165,14 @@ bool SessionImpl::FindStartupConfigFile(ConfigurationScope scope, PathName& path
     RemoveDirectoryDelimiter(bindir.GetData());
     if (Utils::GetPathNamePrefix(myloc, bindir, prefix))
     {
-      path = prefix / MIKTEX_PATH_STARTUP_CONFIG_FILE;
+      path = prefix / PathName(MIKTEX_PATH_STARTUP_CONFIG_FILE);
       if (File::Exists(path))
       {
         return true;
       }
     }
     // try /var/lib/miktex-texmf/miktex/config/miktexstartup.ini
-    path = defaultStartupConfig.commonConfigRoot / MIKTEX_PATH_STARTUP_CONFIG_FILE;
+    path = defaultStartupConfig.commonConfigRoot / PathName(MIKTEX_PATH_STARTUP_CONFIG_FILE);
     if (File::Exists(path))
     {
       return true;
@@ -180,7 +180,7 @@ bool SessionImpl::FindStartupConfigFile(ConfigurationScope scope, PathName& path
 #if defined(MIKTEX_UNIX) && !defined(MIKTEX_MACOS_BUNDLE)
     // try /usr/share/miktex-texmf/miktex/config/miktexstartup.ini
     prefix = GetMyPrefix(true);
-    path = prefix / MIKTEX_INSTALL_DIR / MIKTEX_PATH_STARTUP_CONFIG_FILE;
+    path = prefix / PathName(MIKTEX_INSTALL_DIR) / PathName(MIKTEX_PATH_STARTUP_CONFIG_FILE);
     if (File::Exists(path))
     {
       return true;
@@ -191,7 +191,7 @@ bool SessionImpl::FindStartupConfigFile(ConfigurationScope scope, PathName& path
   if (scope == ConfigurationScope::User)
   {
     // try $HOME/.miktex/miktex/config/miktexstartup.ini
-    path = defaultStartupConfig.userConfigRoot / MIKTEX_PATH_STARTUP_CONFIG_FILE;
+    path = defaultStartupConfig.userConfigRoot / PathName(MIKTEX_PATH_STARTUP_CONFIG_FILE);
     if (File::Exists(path))
     {
       return true;
@@ -217,16 +217,16 @@ pair<bool, PathName> SessionImpl::TryGetBinDirectory(bool canonicalized)
   auto distRoot = TryGetDistRootDirectory();
   if (distRoot.first)
   {
-    return make_pair<bool, PathName>(true, distRoot.second / MIKTEX_PATH_BIN_DIR);
+    return make_pair<bool, PathName>(true, distRoot.second / PathName(MIKTEX_PATH_BIN_DIR));
   }
   string env;
   if (!Utils::GetEnvironmentString(MIKTEX_ENV_BIN_DIR, env))
   {
     return make_pair<bool, PathName>(true, PathName());
   }
-  return make_pair<bool, PathName>(true, env);
+  return make_pair<bool, PathName>(true, PathName(env));
 #elif defined(MIKTEX_MACOS_BUNDLE)
-  return make_pair<bool, PathName>(true, GetMyPrefix(canonicalized) / MIKTEX_BINARY_DESTINATION_DIR);
+  return make_pair<bool, PathName>(true, GetMyPrefix(canonicalized) / PathName(MIKTEX_BINARY_DESTINATION_DIR));
 #else
   return make_pair<bool, PathName>(true, GetMyLocation(canonicalized));
 #endif
@@ -244,7 +244,7 @@ PathName SessionImpl::GetBinDirectory(bool canonicalized)
 
 void SessionImpl::ReadAllConfigFiles(const string& baseName, Cfg& cfg)
 {
-  PathName fileName = MIKTEX_PATH_MIKTEX_CONFIG_DIR / baseName;
+  PathName fileName = PathName(MIKTEX_PATH_MIKTEX_CONFIG_DIR) / PathName(baseName);
   fileName.AppendExtension(".ini");
   vector<PathName> configFiles;
   if (!FindFile(fileName.ToString(), MIKTEX_PATH_TEXMF_PLACEHOLDER, { FindFileOption::All }, configFiles))
@@ -289,7 +289,7 @@ bool SessionImpl::GetSessionValue(const string& sectionName, const string& value
   }
 
   // iterate over application tags, e.g.: latex;tex;miktex
-  for (CsvList app(applicationNames, PathName::PathNameDelimiter); !haveValue && app; ++app)
+  for (CsvList app(applicationNames, PathNameUtil::PathNameDelimiter); !haveValue && app; ++app)
   {
     MIKTEX_ASSERT(!(*app).empty());
 
@@ -404,7 +404,7 @@ bool SessionImpl::GetSessionValue(const string& sectionName, const string& value
   }
   else if (!haveValue && Utils::EqualsIgnoreCase(valueName, CFG_MACRO_NAME_PROGNAME))
   {
-    CsvList progname(applicationNames, PathName::PathNameDelimiter);
+    CsvList progname(applicationNames, PathNameUtil::PathNameDelimiter);
     MIKTEX_ASSERT(progname && !(*progname).empty());
     value = *progname;
     haveValue = true;
@@ -423,7 +423,7 @@ bool SessionImpl::GetSessionValue(const string& sectionName, const string& value
 #endif
   else if (!haveValue && Utils::EqualsIgnoreCase(valueName, CFG_MACRO_NAME_LOCALFONTDIRS))
   {
-    value = StringUtil::Flatten(SessionImpl::GetSession()->GetFontDirectories(), PathName::PathNameDelimiter);
+    value = StringUtil::Flatten(SessionImpl::GetSession()->GetFontDirectories(), PathNameUtil::PathNameDelimiter);
     haveValue = true;
   }
   else if (!haveValue && Utils::EqualsIgnoreCase(valueName, CFG_MACRO_NAME_PSFONTDIRS))
@@ -519,7 +519,7 @@ std::string ConfigValue::GetString() const
   case Type::Char:
     return std::string(1, this->c);
   case Type::StringArray:
-    return StringUtil::Flatten(this->sa, PathName::PathNameDelimiter);
+    return StringUtil::Flatten(this->sa, PathNameUtil::PathNameDelimiter);
   case Type::None:
     MIKTEX_FATAL_ERROR_2(T_("Configuration error: no conversion from undefined configuration value to string."));
   default:
@@ -764,7 +764,7 @@ vector<string> ConfigValue::GetStringArray() const
   switch (type)
   {
   case Type::String:
-    return StringUtil::Split(this->s, PathName::PathNameDelimiter);
+    return StringUtil::Split(this->s, PathNameUtil::PathNameDelimiter);
   case Type::StringArray:
     return this->sa;
   case Type::None:
@@ -896,9 +896,9 @@ bool SessionImpl::IsSharedSetup()
 #else
         PathName myLoc = GetMyLocation(true);
 #if defined(MIKTEX_MACOS_BUNDLE)
-        isSharedSetup = Utils::IsParentDirectoryOf("/usr", myLoc) || Utils::IsParentDirectoryOf("/Applications", myLoc) ? TriState::True : TriState::False;
+        isSharedSetup = Utils::IsParentDirectoryOf(PathName("/usr"), myLoc) || Utils::IsParentDirectoryOf(PathName("/Applications"), myLoc) ? TriState::True : TriState::False;
 #else
-        isSharedSetup = Utils::IsParentDirectoryOf("/usr", myLoc) || Utils::IsParentDirectoryOf("/opt", myLoc) ? TriState::True : TriState::False;
+        isSharedSetup = Utils::IsParentDirectoryOf(PathName("/usr"), myLoc) || Utils::IsParentDirectoryOf(PathName("/opt"), myLoc) ? TriState::True : TriState::False;
 #endif
 #endif
       }
@@ -979,8 +979,8 @@ void SessionImpl::ConfigureFile(const PathName& pathIn, const PathName& pathOut,
     attr -= FileAttribute::ReadOnly;
     File::SetAttributes(pathOut, attr);
   }
-  FileStream streamIn(OpenFile(pathIn.GetData(), FileMode::Open, FileAccess::Read, false));
-  FileStream streamOut(OpenFile(pathOut.GetData(), FileMode::Create, FileAccess::Write, false));
+  FileStream streamIn(OpenFile(pathIn, FileMode::Open, FileAccess::Read, false));
+  FileStream streamOut(OpenFile(pathOut, FileMode::Create, FileAccess::Write, false));
   char chr;
   bool readingName = false;
   string name;
@@ -1198,7 +1198,15 @@ ShellCommandMode SessionImpl::GetShellCommandMode()
   }
   else if (shellCommandMode == "Unrestricted")
   {
-    return ShellCommandMode::Unrestricted;
+    if (RunningAsAdministrator() &&
+      !GetConfigValue(MIKTEX_CONFIG_SECTION_CORE, MIKTEX_CONFIG_VALUE_ALLOW_UNRESTRICTED_SUPER_USER).GetBool())
+    {
+      return ShellCommandMode::Restricted;
+    }
+    else
+    {
+      return ShellCommandMode::Unrestricted;
+    }
   }
   else
   {
@@ -1214,28 +1222,33 @@ vector<string> SessionImpl::GetAllowedShellCommands()
 tuple<Session::ExamineCommandLineResult, string, string> SessionImpl::ExamineCommandLine(const string& commandLine)
 {
   Argv argv(commandLine);
-  if (argv.GetArgc() == 0 || string(argv[0]).find_first_of(" \t") != string::npos)
+  if (argv.GetArgc() == 0)
   {
     return make_tuple(ExamineCommandLineResult::SyntaxError, "", "");
   }
-  PathName argv0(argv[0]);
-  vector<string> allowedCommands = GetAllowedShellCommands();
-  ExamineCommandLineResult examineResult = std::find_if(allowedCommands.begin(), allowedCommands.end(), [argv0](const string& cmd) { return argv0 == cmd; }) != allowedCommands.end()
-    ? ExamineCommandLineResult::ProbablySafe
-    : ExamineCommandLineResult::MaybeSafe;
-  MIKTEX_ASSERT(argv0.ToString().find_first_of(" \t") == string::npos);
-  string toBeExecuted = argv[0];
-#if defined(MIKTEX_WINDOWS)
-  const char quoteChar = '"';
-#else
-  const char quoteChar = '\'';
-#endif
-  for (int idx = 1; idx < argv.GetArgc(); ++idx)
+  ExamineCommandLineResult examineResult = ExamineCommandLineResult::MaybeSafe;
+  if (string(argv[0]).find_first_of("\"' \t") == string::npos)
   {
-    toBeExecuted += ' ';
-    toBeExecuted += quoteChar;
-    toBeExecuted += argv[idx];
-    toBeExecuted += quoteChar;
+    PathName argv0(argv[0]);
+    vector<string> allowedCommands = GetAllowedShellCommands();
+    examineResult = std::find_if(allowedCommands.begin(), allowedCommands.end(), [argv0](const string& cmd) { return argv0 == PathName(cmd); }) != allowedCommands.end()
+      ? ExamineCommandLineResult::ProbablySafe
+      : ExamineCommandLineResult::MaybeSafe;
   }
-  return make_tuple(examineResult, argv[0], toBeExecuted);
+  string safeCommandLine;
+  if (examineResult == ExamineCommandLineResult::ProbablySafe)
+  {
+#if defined(MIKTEX_WINDOWS)
+    const char quoteChar = '"';
+#else
+    const char quoteChar = '\'';
+#endif
+    MIKTEX_ASSERT(string(argv[0]).find_first_of("\"' \t") == string::npos);
+    safeCommandLine = argv[0];
+    for (int idx = 1; idx < argv.GetArgc(); ++idx)
+    {
+      safeCommandLine += fmt::format(" {0}{1}{0}", quoteChar, argv[idx]);
+    }
+  }
+  return make_tuple(examineResult, argv[0], safeCommandLine);
 }
