@@ -84,7 +84,7 @@ static PathName ExpandEnvironmentVariables(const PathName& toBeExpanded)
       expansion += lpsz[0];
     }
   }
-  return expansion;
+  return PathName(expansion);
 }
 
 unsigned SessionImpl::RegisterRootDirectory(const PathName& root, RootDirectoryInfo::Purpose purpose, ConfigurationScope scope, bool other, bool review)
@@ -162,11 +162,11 @@ void SessionImpl::InitializeRootDirectories(const StartupConfig& startupConfig,
     }
 
     // UserRoots
-    for (const string& root : StringUtil::Split(startupConfig.userRoots, PathName::PathNameDelimiter))
+    for (const string& root : StringUtil::Split(startupConfig.userRoots, PathNameUtil::PathNameDelimiter))
     {
       if (!root.empty())
       {
-        RegisterRootDirectory(root, RootDirectoryInfo::Purpose::Generic, ConfigurationScope::User, false, review);
+        RegisterRootDirectory(PathName(root), RootDirectoryInfo::Purpose::Generic, ConfigurationScope::User, false, review);
       }
     }
 
@@ -190,11 +190,11 @@ void SessionImpl::InitializeRootDirectories(const StartupConfig& startupConfig,
   }
 
   // CommonRoots
-  for (const string& root : StringUtil::Split(startupConfig.commonRoots, PathName::PathNameDelimiter))
+  for (const string& root : StringUtil::Split(startupConfig.commonRoots, PathNameUtil::PathNameDelimiter))
   {
     if (!root.empty())
     {
-      RegisterRootDirectory(root, RootDirectoryInfo::Purpose::Generic, ConfigurationScope::Common, false, review);
+      RegisterRootDirectory(PathName(root), RootDirectoryInfo::Purpose::Generic, ConfigurationScope::Common, false, review);
     }
   }
 
@@ -207,21 +207,21 @@ void SessionImpl::InitializeRootDirectories(const StartupConfig& startupConfig,
   if (!IsAdminMode())
   {
     // OtherUserRoots
-    for (const string& root : StringUtil::Split(startupConfig.otherUserRoots, PathName::PathNameDelimiter))
+    for (const string& root : StringUtil::Split(startupConfig.otherUserRoots, PathNameUtil::PathNameDelimiter))
     {
       if (!root.empty())
       {
-        RegisterRootDirectory(root, RootDirectoryInfo::Purpose::Generic, ConfigurationScope::User, true, review);
+        RegisterRootDirectory(PathName(root), RootDirectoryInfo::Purpose::Generic, ConfigurationScope::User, true, review);
       }
     }
   }
 
   // OtherCommonRoots
-  for (const string& root : StringUtil::Split(startupConfig.otherCommonRoots, PathName::PathNameDelimiter))
+  for (const string& root : StringUtil::Split(startupConfig.otherCommonRoots, PathNameUtil::PathNameDelimiter))
   {
     if (!root.empty())
     {
-      RegisterRootDirectory(root, RootDirectoryInfo::Purpose::Generic, ConfigurationScope::Common, true, review);
+      RegisterRootDirectory(PathName(root), RootDirectoryInfo::Purpose::Generic, ConfigurationScope::Common, true, review);
     }
   }
 
@@ -261,7 +261,7 @@ void SessionImpl::InitializeRootDirectories(const StartupConfig& startupConfig,
     commonInstallRootIndex = commonConfigRootIndex;
   }
 
-  RegisterRootDirectory(MPM_ROOT_PATH, RootDirectoryInfo::Purpose::Generic, IsAdminMode() ? ConfigurationScope::Common : ConfigurationScope::User, false, false);
+  RegisterRootDirectory(PathName(MPM_ROOT_PATH), RootDirectoryInfo::Purpose::Generic, IsAdminMode() ? ConfigurationScope::Common : ConfigurationScope::User, false, false);
 
   if (!IsAdminMode())
   {
@@ -379,7 +379,7 @@ pair<bool, PathName> SessionImpl::TryGetDistRootDirectory()
   }
   return make_pair(false, PathName());
 #else
-  return make_pair(true, GetMyPrefix(true) / MIKTEX_DIST_DIR);
+  return make_pair(true, GetMyPrefix(true) / PathName(MIKTEX_DIST_DIR));
 #endif
 }
 
@@ -471,7 +471,7 @@ void SessionImpl::RegisterRootDirectory(const PathName& path, bool other)
     }
   }
   toBeRegistered.push_back(path.ToString());
-  ReregisterRootDirectories(StringUtil::Flatten(toBeRegistered, PathName::PathNameDelimiter), other);
+  ReregisterRootDirectories(StringUtil::Flatten(toBeRegistered, PathNameUtil::PathNameDelimiter), other);
 }
 
 void SessionImpl::UnregisterRootDirectory(const PathName& path, bool other)
@@ -502,7 +502,7 @@ void SessionImpl::UnregisterRootDirectory(const PathName& path, bool other)
   {
     MIKTEX_UNEXPECTED();
   }
-  ReregisterRootDirectories(StringUtil::Flatten(toBeRegistered, PathName::PathNameDelimiter), other);
+  ReregisterRootDirectories(StringUtil::Flatten(toBeRegistered, PathNameUtil::PathNameDelimiter), other);
 }
 
 void SessionImpl::RegisterRootDirectories(const StartupConfig& partialStartupConfig, RegisterRootDirectoriesOptionSet options)
@@ -632,7 +632,7 @@ void SessionImpl::MoveRootDirectory(unsigned r, int dir)
       toBeRegistered.push_back(root.path.ToString());
     }
   }
-  ReregisterRootDirectories(StringUtil::Flatten(toBeRegistered, PathName::PathNameDelimiter), false);
+  ReregisterRootDirectories(StringUtil::Flatten(toBeRegistered, PathNameUtil::PathNameDelimiter), false);
 }
 
 void SessionImpl::MoveRootDirectoryUp(unsigned r)
@@ -764,12 +764,12 @@ vector<PathName> SessionImpl::GetFilenameDatabasePathNames(unsigned r)
     {
       MIKTEX_UNEXPECTED();
     }
-    path = rootDirectories[GetInstallRoot()].get_Path() / MIKTEX_PATH_MPM_FNDB;
+    path = rootDirectories[GetInstallRoot()].get_Path() / PathName(MIKTEX_PATH_MPM_FNDB);
   }
   else
   {
     // ROOT\miktex\conig\texmf.fndb
-    path = rootDirectories[r].get_Path() / MIKTEX_PATH_TEXMF_FNDB;
+    path = rootDirectories[r].get_Path() / PathName(MIKTEX_PATH_TEXMF_FNDB);
   }
   result.push_back(path);
 
@@ -783,13 +783,13 @@ PathName SessionImpl::GetMpmDatabasePathName()
 
 PathName SessionImpl::GetMpmRootPath()
 {
-  return MPM_ROOT_PATH;
+  return PathName(MPM_ROOT_PATH);
 }
 
 PathName SessionImpl::GetRelativeFilenameDatabasePathName(unsigned r)
 {
   string fndbFileName = MIKTEX_PATH_FNDB_DIR;
-  fndbFileName += PathName::DirectoryDelimiter;
+  fndbFileName += PathNameUtil::DirectoryDelimiter;
   PathName root(rootDirectories[r].get_Path());
   root.TransformForComparison();
   MD5Builder md5Builder;
@@ -797,7 +797,7 @@ PathName SessionImpl::GetRelativeFilenameDatabasePathName(unsigned r)
   md5Builder.Final();
   fndbFileName += md5Builder.GetMD5().ToString();
   fndbFileName += MIKTEX_FNDB_FILE_SUFFIX;
-  return fndbFileName;
+  return PathName(fndbFileName);
 }
 
 shared_ptr<FileNameDatabase> SessionImpl::GetFileNameDatabase(unsigned r)
@@ -851,7 +851,7 @@ shared_ptr<FileNameDatabase> SessionImpl::GetFileNameDatabase(unsigned r)
 
 shared_ptr<FileNameDatabase> SessionImpl::GetFileNameDatabase(const char* path)
 {
-  unsigned root = TryDeriveTEXMFRoot(path);
+  unsigned root = TryDeriveTEXMFRoot(PathName(path));
   if (root == INVALID_ROOT_INDEX)
   {
     return nullptr;
@@ -861,7 +861,7 @@ shared_ptr<FileNameDatabase> SessionImpl::GetFileNameDatabase(const char* path)
 
 unsigned SessionImpl::TryDeriveTEXMFRoot(const PathName& path)
 {
-  if (!Utils::IsAbsolutePath(path))
+  if (!path.IsFullyQUalified())
   {
 #if FIND_FILE_PREFER_RELATIVE_PATH_NAMES
     return INVALID_ROOT_INDEX;
@@ -883,7 +883,7 @@ unsigned SessionImpl::TryDeriveTEXMFRoot(const PathName& path)
   {
     PathName pathRoot = GetRootDirectoryPath(idx);
     size_t rootlen = pathRoot.GetLength();
-    if (PathName::Compare(pathRoot, path, rootlen) == 0 && (pathRoot.EndsWithDirectoryDelimiter() || path[rootlen] == 0 || IsDirectoryDelimiter(path[rootlen])))
+    if (PathName::Compare(pathRoot, path, rootlen) == 0 && (pathRoot.EndsWithDirectoryDelimiter() || path[rootlen] == 0 || PathNameUtil::IsDirectoryDelimiter(path[rootlen])))
     {
       if (rootDirectoryIndex == INVALID_ROOT_INDEX)
       {
@@ -966,10 +966,10 @@ bool SessionImpl::IsTEXMFFile(const PathName& path, PathName& relPath, unsigned&
   {
     PathName pathRoot = GetRootDirectoryPath(r);
     size_t cchRoot = pathRoot.GetLength();
-    if (PathName::Compare(pathRoot, path, cchRoot) == 0 && (path[cchRoot] == 0 || IsDirectoryDelimiter(path[cchRoot])))
+    if (PathName::Compare(pathRoot, path, cchRoot) == 0 && (path[cchRoot] == 0 || PathNameUtil::IsDirectoryDelimiter(path[cchRoot])))
     {
       const char* lpsz = &path[cchRoot];
-      if (IsDirectoryDelimiter(*lpsz))
+      if (PathNameUtil::IsDirectoryDelimiter(*lpsz))
       {
         ++lpsz;
       }
@@ -987,12 +987,12 @@ unsigned SessionImpl::SplitTEXMFPath(const PathName& path, PathName& root, PathN
   {
     PathName rootDir = GetRootDirectoryPath(r);
     size_t rootDirLen = rootDir.GetLength();
-    if (PathName::Compare(rootDir, path, rootDirLen) == 0 && (path[rootDirLen] == 0 || IsDirectoryDelimiter(path[rootDirLen])))
+    if (PathName::Compare(rootDir, path, rootDirLen) == 0 && (path[rootDirLen] == 0 || PathNameUtil::IsDirectoryDelimiter(path[rootDirLen])))
     {
       root = rootDir;
       root[rootDirLen] = 0;
       const char* lpsz = &path[0] + rootDirLen;
-      if (IsDirectoryDelimiter(*lpsz))
+      if (PathNameUtil::IsDirectoryDelimiter(*lpsz))
       {
         ++lpsz;
       }
@@ -1018,7 +1018,7 @@ bool SessionImpl::IsManagedRoot(unsigned root)
 bool SessionImpl::IsMpmFile(const char* lpszPath)
 {
   return (PathName::Compare(MPM_ROOT_PATH, lpszPath, static_cast<unsigned long>(MPM_ROOT_PATH_LEN)) == 0
-    && (lpszPath[MPM_ROOT_PATH_LEN] == 0 || IsDirectoryDelimiter(lpszPath[MPM_ROOT_PATH_LEN])));
+    && (lpszPath[MPM_ROOT_PATH_LEN] == 0 || PathNameUtil::IsDirectoryDelimiter(lpszPath[MPM_ROOT_PATH_LEN])));
 }
 
 bool Utils::IsMiKTeXDirectRoot(const PathName& root)
