@@ -43,6 +43,7 @@
 using namespace std;
 
 using namespace MiKTeX::Core;
+using namespace MiKTeX::Util;
 
 void SessionImpl::MyCoInitialize()
 {
@@ -264,12 +265,12 @@ StartupConfig SessionImpl::DefaultConfig(MiKTeXConfiguration config, const PathN
     if (!commonPrefix.Empty())
     {
       PathName portableRoot;
-      bool isLegacy = !Utils::GetPathNamePrefix(commonPrefix, MIKTEX_PORTABLE_REL_INSTALL_DIR, portableRoot);
+      bool isLegacy = !Utils::GetPathNamePrefix(commonPrefix, PathName(MIKTEX_PORTABLE_REL_INSTALL_DIR), portableRoot);
       ret.commonInstallRoot = commonPrefix;
       if (!isLegacy)
       {
-        ret.commonConfigRoot = portableRoot / MIKTEX_PORTABLE_REL_CONFIG_DIR;
-        ret.commonDataRoot = portableRoot / MIKTEX_PORTABLE_REL_DATA_DIR;
+        ret.commonConfigRoot = portableRoot / PathName(MIKTEX_PORTABLE_REL_CONFIG_DIR);
+        ret.commonDataRoot = portableRoot / PathName(MIKTEX_PORTABLE_REL_DATA_DIR);
       }
       else
       {
@@ -280,12 +281,12 @@ StartupConfig SessionImpl::DefaultConfig(MiKTeXConfiguration config, const PathN
     if (!userPrefix.Empty())
     {
       PathName portableRoot;
-      bool isLegacy = !Utils::GetPathNamePrefix(userPrefix, MIKTEX_PORTABLE_REL_INSTALL_DIR, portableRoot);
+      bool isLegacy = !Utils::GetPathNamePrefix(userPrefix, PathName(MIKTEX_PORTABLE_REL_INSTALL_DIR), portableRoot);
       ret.userInstallRoot = userPrefix;
       if (!isLegacy)
       {
-        ret.userConfigRoot = portableRoot / MIKTEX_PORTABLE_REL_CONFIG_DIR;
-        ret.userDataRoot = portableRoot / MIKTEX_PORTABLE_REL_DATA_DIR;
+        ret.userConfigRoot = portableRoot / PathName(MIKTEX_PORTABLE_REL_CONFIG_DIR);
+        ret.userDataRoot = portableRoot / PathName(MIKTEX_PORTABLE_REL_DATA_DIR);
       }
       else
       {
@@ -297,19 +298,19 @@ StartupConfig SessionImpl::DefaultConfig(MiKTeXConfiguration config, const PathN
     {
       PathName myloc(GetMyLocation(false));
       PathName prefix;
-      if (Utils::GetPathNamePrefix(myloc, MIKTEX_PATH_INTERNAL_BIN_DIR, prefix)
-        || Utils::GetPathNamePrefix(myloc, MIKTEX_PATH_BIN_DIR, prefix)
-        || Utils::GetPathNamePrefix(myloc, MIKTEX_PATH_MIKTEX_TEMP_DIR, prefix))
+      if (Utils::GetPathNamePrefix(myloc, PathName(MIKTEX_PATH_INTERNAL_BIN_DIR), prefix)
+        || Utils::GetPathNamePrefix(myloc, PathName(MIKTEX_PATH_BIN_DIR), prefix)
+        || Utils::GetPathNamePrefix(myloc, PathName(MIKTEX_PATH_MIKTEX_TEMP_DIR), prefix))
       {
         PathName portableRoot;
-        bool isLegacy = !Utils::GetPathNamePrefix(prefix, MIKTEX_PORTABLE_REL_INSTALL_DIR, portableRoot);
+        bool isLegacy = !Utils::GetPathNamePrefix(prefix, PathName(MIKTEX_PORTABLE_REL_INSTALL_DIR), portableRoot);
         if (commonPrefix.Empty())
         {
           ret.commonInstallRoot = prefix;
           if (!isLegacy)
           {
-            ret.commonConfigRoot = portableRoot / MIKTEX_PORTABLE_REL_CONFIG_DIR;
-            ret.commonDataRoot = portableRoot / MIKTEX_PORTABLE_REL_DATA_DIR;
+            ret.commonConfigRoot = portableRoot / PathName(MIKTEX_PORTABLE_REL_CONFIG_DIR);
+            ret.commonDataRoot = portableRoot / PathName(MIKTEX_PORTABLE_REL_DATA_DIR);
           }
           else
           {
@@ -322,8 +323,8 @@ StartupConfig SessionImpl::DefaultConfig(MiKTeXConfiguration config, const PathN
           ret.userInstallRoot = prefix;
           if (!isLegacy)
           {
-            ret.userConfigRoot = portableRoot / MIKTEX_PORTABLE_REL_CONFIG_DIR;
-            ret.userDataRoot = portableRoot / MIKTEX_PORTABLE_REL_DATA_DIR;
+            ret.userConfigRoot = portableRoot / PathName(MIKTEX_PORTABLE_REL_CONFIG_DIR);
+            ret.userDataRoot = portableRoot / PathName(MIKTEX_PORTABLE_REL_DATA_DIR);
           }
           else
           {
@@ -342,7 +343,7 @@ StartupConfig SessionImpl::DefaultConfig(MiKTeXConfiguration config, const PathN
       product = "MiKTeXDirect";
       PathName myloc(GetMyLocation(false));
       PathName prefix;
-      if (!Utils::GetPathNamePrefix(myloc, MIKTEX_PATH_BIN_DIR, prefix))
+      if (!Utils::GetPathNamePrefix(myloc, PathName(MIKTEX_PATH_BIN_DIR), prefix))
       {
         MIKTEX_UNEXPECTED();
       }
@@ -564,11 +565,11 @@ bool SessionImpl::GetAcrobatFontDir(PathName& path)
     PathName fontDir;
 
     // try Acrobat Reader 3.0
-    fontDir = dir / "FONTS";
+    fontDir = dir / PathName("FONTS");
     if (!Directory::Exists(fontDir))
     {
       // try Acrobat Reader 4.0
-      fontDir = dir / ".." / "Resource" / "Font";
+      fontDir = dir / PathName("..") / PathName("Resource") / PathName("Font");
       if (!Directory::Exists(fontDir))
       {
         return false;
@@ -577,7 +578,7 @@ bool SessionImpl::GetAcrobatFontDir(PathName& path)
 
     RemoveDirectoryDelimiter(fontDir.GetData());
 
-    acrobatFontDir = GetFullPath(fontDir.GetData());
+    acrobatFontDir = GetFullyQualifiedPath(fontDir.GetData());
   }
 
   if (acrobatFontDir.GetLength() == 0)
@@ -614,7 +615,7 @@ bool SessionImpl::GetATMFontDir(PathName& path)
 
     RemoveDirectoryDelimiter(fontDir.GetData());
 
-    atmFontDir = GetFullPath(fontDir.GetData());
+    atmFontDir = GetFullyQualifiedPath(fontDir.GetData());
   }
 
   if (atmFontDir.GetLength() == 0)
@@ -636,7 +637,7 @@ MIKTEXSTATICFUNC(bool) GetPsFontDirectory(PathName& path)
     MIKTEX_FATAL_WINDOWS_ERROR("GetWindowsDirectoryW");
   }
 
-  PathName path_ = PathName(szWinDir) / "psfonts";
+  PathName path_ = PathName(szWinDir) / PathName("psfonts");
 
   if (!Directory::Exists(path_))
   {
@@ -658,7 +659,7 @@ bool SessionImpl::GetPsFontDirs(string& psFontDirs)
     {
       if (!this->psFontDirs.empty())
       {
-        this->psFontDirs += PathName::PathNameDelimiter;
+        this->psFontDirs += PathNameUtil::PathNameDelimiter;
       }
       this->psFontDirs += path.GetData();
     }
@@ -666,7 +667,7 @@ bool SessionImpl::GetPsFontDirs(string& psFontDirs)
     {
       if (!this->psFontDirs.empty())
       {
-        this->psFontDirs += PathName::PathNameDelimiter;
+        this->psFontDirs += PathNameUtil::PathNameDelimiter;
       }
       this->psFontDirs += path.GetData();
     }
@@ -692,7 +693,7 @@ bool SessionImpl::GetTTFDirs(string& ttfDirs)
     {
       if (!this->ttfDirs.empty())
       {
-        this->ttfDirs += PathName::PathNameDelimiter;;
+        this->ttfDirs += PathNameUtil::PathNameDelimiter;;
       }
       this->ttfDirs += path.GetData();
     }
@@ -718,7 +719,7 @@ bool SessionImpl::GetOTFDirs(string& otfDirs)
     {
       if (!this->otfDirs.empty())
       {
-        this->otfDirs += PathName::PathNameDelimiter;
+        this->otfDirs += PathNameUtil::PathNameDelimiter;
       }
       this->otfDirs += path.GetData();
     }
@@ -801,7 +802,7 @@ bool SessionImpl::ShowManualPageAndWait(HWND hWnd, unsigned long topic)
   {
     return false;
   }
-  HWND hwnd = HtmlHelpW(hWnd, pathHelpFile.ToWideCharString().c_str(), HH_HELP_CONTEXT, topic);
+  HWND hwnd = HtmlHelpW(hWnd, pathHelpFile.ToExtendedLengthPathName().ToWideCharString().c_str(), HH_HELP_CONTEXT, topic);
   if (hwnd == nullptr)
   {
     return false;
@@ -821,7 +822,7 @@ bool SessionImpl::IsFileAlreadyOpen(const PathName& fileName)
 {
   unsigned long error = NO_ERROR;
 
-  HANDLE hFile = CreateFileW(fileName.ToWideCharString().c_str(), GENERIC_READ, FILE_SHARE_READ, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, nullptr);
+  HANDLE hFile = CreateFileW(fileName.ToExtendedLengthPathName().ToWideCharString().c_str(), GENERIC_READ, FILE_SHARE_READ, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, nullptr);
 
   if (hFile == INVALID_HANDLE_VALUE)
   {
