@@ -26,6 +26,9 @@
 #include <strsafe.h>
 #include <wininet.h>
 
+#include <fmt/format.h>
+#include <fmt/ostream.h>
+
 #include <miktex/Core/Directory>
 #include <miktex/Core/PathName>
 #include <miktex/Core/Paths>
@@ -43,27 +46,6 @@ using namespace MiKTeX::Core;
 using namespace MiKTeX::Trace;
 using namespace MiKTeX::Util;
 
-MIKTEXSTATICFUNC(void) GetAlternate(const char* lpszPath, char* lpszAlternate)
-{
-  MIKTEX_ASSERT_STRING(lpszPath);
-  MIKTEX_ASSERT_PATH_BUFFER(lpszAlternate);
-  WIN32_FIND_DATAW finddata;
-  HANDLE hnd = FindFirstFileW(PathName(lpszPath).ToWideCharString().c_str(), &finddata);
-  if (hnd == INVALID_HANDLE_VALUE)
-  {
-    MIKTEX_FATAL_WINDOWS_ERROR_2("FindFirstFileW", "path", lpszPath);
-  }
-  if (!FindClose(hnd))
-  {
-    MIKTEX_FATAL_WINDOWS_ERROR("FindClose");
-  }
-  if (finddata.cAlternateFileName[0] == 0)
-  {
-    MIKTEX_FATAL_ERROR_2(T_("No alternate file name found."), "path", lpszPath);
-  }
-  StringUtil::CopyString(lpszAlternate, BufferSizes::MaxPath, finddata.cAlternateFileName);
-}
-
 PathName Utils::GetFolderPath(int nFolder, int nFallbackFolder, bool getCurrentPath)
 {
   DWORD flags = getCurrentPath ? SHGFP_TYPE_CURRENT : SHGFP_TYPE_DEFAULT;
@@ -89,102 +71,7 @@ PathName Utils::GetFolderPath(int nFolder, int nFallbackFolder, bool getCurrentP
   {
     MIKTEX_FATAL_ERROR_2(T_("A required file system path could not be retrieved."), "nfolder", std::to_string(nFolder), "hr", std::to_string(hr));
   }
-  return szPath;
-}
-
-void Utils::RemoveBlanksFromPathName(PathName& path)
-{
-  // make a working copy
-  PathName temp(path);
-
-  // points to the null char
-  char* lpszEnd = temp.GetData() + temp.GetLength();
-
-  // back iterator
-  char* lpsz = lpszEnd;
-
-  // points to the last '/' found (if any)
-  char* lpszSlash = nullptr;
-
-  // true, if lpsz points to a char sequence that includes a blank
-  // char
-  bool containsBlanks = false;
-
-  // find the last slash
-  while (lpsz != temp.GetData())
-  {
-    --lpsz;
-    if (IsDirectoryDelimiter(*lpsz) && lpsz + 1 != lpszEnd)
-    {
-      lpszSlash = lpsz;
-      break;
-    }
-    if (*lpsz == ' ')
-    {
-      containsBlanks = true;
-    }
-  }
-
-  // done, if we're at the beginning of the path and if there is no
-  // blank
-  if (!containsBlanks && lpsz == temp.GetData())
-  {
-    return;
-  }
-
-  // lpszSlash points to the last '/'
-  MIKTEX_ASSERT(lpszSlash == nullptr || IsDirectoryDelimiter(*lpszSlash));
-
-  // buffer for file name w/o blanks
-  char szAlternate[BufferSizes::MaxPath];
-
-  if (containsBlanks)
-  {
-    // get short file name (8.3 w/o blanks)
-    GetAlternate(temp.GetData(), szAlternate);
-  }
-  else
-  {
-    // no blanks found in original file name, i.e., keep the
-    // original file name
-    StringUtil::CopyString(szAlternate, BufferSizes::MaxPath, (lpsz == lpszSlash ? lpsz + 1 : lpsz));
-  }
-
-  // the new file name doesn't contain a space
-  MIKTEX_ASSERT(StrChr(szAlternate, ' ') == nullptr);
-
-  if (lpszSlash == temp.GetData()   // "/foo.bar"
-    || (lpszSlash == temp.GetData() + 1 // "//foo.bar"
-      && PathName::IsDirectoryDelimiter(temp[0]))
-    || (lpszSlash == temp.GetData() + 2   // "C:/foo.bar"
-      && IsAlpha(temp[0])
-      && PathName::IsVolumeDelimiter(temp[1])))
-  {
-    // we have reached the beginning of the path; separate original
-    // file name from path; then combine path with new file name
-    lpszSlash[1] = 0;
-    path = temp;
-    path /= szAlternate;
-  }
-  else if (lpszSlash != nullptr)
-  {
-    // separate file name from path
-    *lpszSlash = 0;
-    // still have to remove blanks from path
-    // <recursivecall>
-    RemoveBlanksFromPathName(temp);
-    // </recursivecall>
-    // path now hasn't any blanks; combine path with new file name
-    MIKTEX_ASSERT(StrChr(temp.GetData(), ' ') == nullptr);
-    path = temp;
-    path /= szAlternate;
-  }
-  else
-  {
-    // original path was something like 'foo bar'; simply turn back
-    // the new version (something like 'FOOBAR~1')
-    path = szAlternate;
-  }
+  return PathName(szPath);
 }
 
 string GetOperatingSystem(const OSVERSIONINFOEXW& osvi, const SYSTEM_INFO& si)
@@ -825,7 +712,7 @@ void Utils::SetEnvironmentString(const string& valueName, const string& value)
   shared_ptr<SessionImpl> session = SessionImpl::TryGetSession();
   if (session != nullptr)
   {
-    session->trace_config->WriteFormattedLine("core", T_("setting env %s=%s"), valueName.c_str(), value.c_str());
+    session->trace_config->WriteLine("core", fmt::format(T_("setting env {0}={1}"), valueName, value));
   }
 #if defined(_MSC_VER) || defined(__MINGW32__)
   if (_wputenv_s(UW_(valueName), UW_(value)) != 0)
@@ -1175,7 +1062,7 @@ bool Utils::CheckPath(bool repair)
     }
   }
 
-  PathName commonBinDir = session->GetSpecialPath(SpecialPath::CommonInstallRoot) / MIKTEX_PATH_BIN_DIR;
+  PathName commonBinDir = session->GetSpecialPath(SpecialPath::CommonInstallRoot) / PathName(MIKTEX_PATH_BIN_DIR);
 
   string repairedSystemPath;
   bool systemPathCompetition;
@@ -1218,7 +1105,7 @@ bool Utils::CheckPath(bool repair)
         repaired = true;
       }
     }
-    PathName userBinDir = session->GetSpecialPath(SpecialPath::UserInstallRoot) / MIKTEX_PATH_BIN_DIR;
+    PathName userBinDir = session->GetSpecialPath(SpecialPath::UserInstallRoot) / PathName(MIKTEX_PATH_BIN_DIR);
     string repairedUserPath;
     bool userPathCompetition;
     userPathOkay = !Directory::Exists(userBinDir) || !FixProgramSearchPath(WU_(userPath), userBinDir, true, repairedUserPath, userPathCompetition);
@@ -1250,17 +1137,29 @@ bool Utils::CheckPath(bool repair)
 
 void Utils::CanonicalizePathName(PathName& path)
 {
-  wchar_t szFullPath[BufferSizes::MaxPath];
-  DWORD n = GetFullPathNameW(path.ToWideCharString().c_str(), BufferSizes::MaxPath, szFullPath, nullptr);
-  if (n == 0)
+  CharBuffer<wchar_t, BufferSizes::MaxPath> fullPath;
+  bool done = false;
+  int rounds = 0;
+  do
   {
-    MIKTEX_FATAL_WINDOWS_ERROR_2("GetFullPathNameW", "path", path.ToString());
+    DWORD n = GetFullPathNameW(path.ToWideCharString().c_str(), fullPath.GetCapacity(), fullPath.GetData(), nullptr);
+    if (n == 0)
+    {
+      MIKTEX_FATAL_WINDOWS_ERROR_2("GetFullPathNameW", "path", path.ToString());
+    }
+    done = n < fullPath.GetCapacity();
+    if (!done)
+    {
+      if (rounds > 0)
+      {
+        BUF_TOO_SMALL();
+      }
+      fullPath.Reserve(n);
+    }
+    rounds++;
   }
-  if (n >= BufferSizes::MaxPath)
-  {
-    BUF_TOO_SMALL();
-  }
-  path = szFullPath;
+  while (!done);
+  path = fullPath.GetData();
 }
 
 void Utils::RegisterShellFileAssoc(const string& extension, const string& progId, bool takeOwnership)
