@@ -143,19 +143,19 @@ MIKTEXINTERNALFUNC(void) TraceWindowsError(const char* windowsFunction, unsigned
 MIKTEXSTATICFUNC(unsigned int) GetMediaType(const char* path)
 {
   PathName pathRootName;
-  if (IsAlpha(path[0])
-    && PathName::IsVolumeDelimiter(path[1])
-    && PathName::IsDirectoryDelimiter(path[2]))
+  if (PathNameUtil::IsDosDriveLetter(path[0])
+    && PathNameUtil::IsDosVolumeDelimiter(path[1])
+    && PathNameUtil::IsDirectoryDelimiter(path[2]))
   {
     pathRootName += path[0];
-    pathRootName += PathName::VolumeDelimiter;
-    pathRootName += PathName::DirectoryDelimiter;
+    pathRootName += PathNameUtil::DosVolumeDelimiter;
+    pathRootName += PathNameUtil::DirectoryDelimiter;
   }
-  else if (!Utils::GetUncRootFromPath(path, pathRootName))
+  else if (!Utils::GetUncRootFromPath(PathName(path), pathRootName))
   {
     return DRIVE_UNKNOWN;
   }
-  return GetDriveTypeW(pathRootName.ToWideCharString().c_str());
+  return GetDriveTypeW(pathRootName.ToExtendedLengthPathName().ToWideCharString().c_str());
 }
 
 MIKTEXINTERNALFUNC(bool) FileIsOnROMedia(const char* path)
@@ -210,7 +210,7 @@ MIKTEXSTATICFUNC(void) CreateDirectoryForEveryone(const char* path)
   sa.lpSecurityDescriptor = pSD;
   sa.bInheritHandle = FALSE;
 
-  if (!CreateDirectoryW(PathName(path).ToWideCharString().c_str(), &sa))
+  if (!CreateDirectoryW(PathName(path).ToExtendedLengthPathName().ToWideCharString().c_str(), &sa))
   {
     MIKTEX_FATAL_WINDOWS_ERROR_2("CreateDirectoryW", "path", path);
   }
@@ -219,12 +219,12 @@ MIKTEXSTATICFUNC(void) CreateDirectoryForEveryone(const char* path)
 
 MIKTEXINTERNALFUNC(void) CreateDirectoryPath(const PathName& path)
 {
-  if (!Utils::IsAbsolutePath(path))
+  if (!path.IsFullyQUalified())
   {
-    PathName absolutePath(path);
-    absolutePath.MakeAbsolute();
+    PathName fqPath(path);
+    fqPath.MakeFullyQualified();
     // RECURSION
-    CreateDirectoryPath(absolutePath);
+    CreateDirectoryPath(fqPath);
   }
 
   // do nothing, if the directory already exists
@@ -249,7 +249,7 @@ MIKTEXINTERNALFUNC(void) CreateDirectoryPath(const PathName& path)
 
   if (session != nullptr)
   {
-    session->trace_files->WriteFormattedLine("core", T_("creating directory %s"), Q_(path));
+    session->trace_files->WriteLine("core", fmt::format(T_("creating directory {0}"), Q_(path)));
   }
 
 #if SET_SECURITY
@@ -260,12 +260,12 @@ MIKTEXINTERNALFUNC(void) CreateDirectoryPath(const PathName& path)
   {
     CreateDirectoryForEveryone(path.GetData());
   }
-  else if (!CreateDirectoryW(path.ToWideCharString().c_str(), nullptr))
+  else if (!CreateDirectoryW(path.ToExtendedLengthPathName().ToWideCharString().c_str(), nullptr))
   {
     MIKTEX_FATAL_WINDOWS_ERROR_2("CreateDirectoryW", "path", path.ToString());
   }
 #else
-  if (!CreateDirectoryW(path.ToWideCharString().c_str(), nullptr))
+  if (!CreateDirectoryW(path.ToExtendedLengthPathName().ToWideCharString().c_str(), nullptr))
   {
     MIKTEX_FATAL_WINDOWS_ERROR_2("CreateDirectoryW", "path", path.ToString());
   }
