@@ -84,13 +84,33 @@ public:
   }
 
 public:
-  void MIKTEXTHISCALL UpdateDb(UpdateDbOptionSet options) override;
+  void MIKTEXTHISCALL UpdateDb(UpdateDbOptionSet options) override
+  {
+    MPM_LOCK_BEGIN(this->packageManager)
+    {
+      UpdateDbNoLock(options);
+    }
+    MPM_LOCK_END();
+  }
+
+private:
+  void UpdateDbNoLock(UpdateDbOptionSet options);
 
 public:
   void MIKTEXTHISCALL UpdateDbAsync() override;
 
 public:
-  void MIKTEXTHISCALL FindUpdates() override;
+  void MIKTEXTHISCALL FindUpdates() override
+  {
+    MPM_LOCK_BEGIN(this->packageManager)
+    {
+      FindUpdatesNoLock();
+    }
+    MPM_LOCK_END();
+  }
+
+private:
+  void FindUpdatesNoLock();
 
 public:
   void MIKTEXTHISCALL FindUpdatesAsync() override;
@@ -102,7 +122,13 @@ public:
   }
 
 public:
-  void MIKTEXTHISCALL FindUpgrades(PackageLevel packageLevel) override;
+  void MIKTEXTHISCALL FindUpgrades(PackageLevel packageLevel) override
+  {
+    FindUpgradesNoLock(packageLevel);
+  }
+
+private:
+  void FindUpgradesNoLock(PackageLevel packageLevel);
 
 public:
   void MIKTEXTHISCALL FindUpgradesAsync(PackageLevel packageLevel) override;
