@@ -27,6 +27,8 @@
 
 #include <miktex/PackageManager/PackageManager>
 
+#include <miktex/Util/PathNameUtil>
+
 #include "internal.h"
 #include "PackageRepositoryDataStore.h"
 #include "RemoteService.h"
@@ -36,6 +38,7 @@ using namespace std;
 
 using namespace MiKTeX::Core;
 using namespace MiKTeX::Packages;
+using namespace MiKTeX::Util;
 
 using namespace MiKTeX::Packages::D6AAD62216146D44B580E92711724B78;
 
@@ -44,8 +47,8 @@ PackageRepositoryDataStore::PackageRepositoryDataStore(std::shared_ptr<WebSessio
 {
   MIKTEX_ASSERT(webSession != nullptr);
   comboCfg.Load(
-    session->IsAdminMode() ? "" : session->GetSpecialPath(SpecialPath::UserConfigRoot) / MIKTEX_PATH_REPOSITORIES_INI,
-    session->GetSpecialPath(SpecialPath::CommonConfigRoot) / MIKTEX_PATH_REPOSITORIES_INI);
+    session->IsAdminMode() ? PathName() : session->GetSpecialPath(SpecialPath::UserConfigRoot) / PathName(MIKTEX_PATH_REPOSITORIES_INI),
+    session->GetSpecialPath(SpecialPath::CommonConfigRoot) / PathName(MIKTEX_PATH_REPOSITORIES_INI));
 }
 
 void PackageRepositoryDataStore::Download()
@@ -128,17 +131,17 @@ RepositoryType PackageRepositoryDataStore::DetermineRepositoryType(const string&
     return RepositoryType::Remote;
   }
 
-  if (!Utils::IsAbsolutePath(repository))
+  if (!PathNameUtil::IsAbsolutePath(repository))
   {
     MIKTEX_UNEXPECTED();
   }
 
-  if (PackageManager::IsLocalPackageRepository(repository))
+  if (PackageManager::IsLocalPackageRepository(PathName(repository)))
   {
     return RepositoryType::Local;
   }
 
-  if (Utils::IsMiKTeXDirectRoot(repository))
+  if (Utils::IsMiKTeXDirectRoot(PathName(repository)))
   {
     return RepositoryType::MiKTeXDirect;
   }
