@@ -84,7 +84,7 @@ protected:
     }
     if (trace_mem->IsEnabled("libtexmf", MiKTeX::Trace::TraceLevel::Trace))
     {
-      trace_mem->WriteFormattedLine("libtexmf", MIKTEXTEXT("Parameter %s: %d"), parameterName.c_str(), result);
+      // TODO: trace_mem->WriteLine("libtexmf", fmt::format(MIKTEXTEXT("Parameter {0}: {1}"), parameterName, result));
     }
     return result;
   }
@@ -108,7 +108,7 @@ protected:
     }
     if (trace_mem->IsEnabled("libtexmf", MiKTeX::Trace::TraceLevel::Trace))
     {
-      trace_mem->WriteFormattedLine("libtexmf", MIKTEXTEXT("Parameter %s: %d"), parameterName.c_str(), result);
+      // TODO: trace_mem->WriteLine("libtexmf", fmt::format(MIKTEXTEXT("Parameter {0}: {1}"), parameterName, result));
     }
     return result;
   }
@@ -278,15 +278,8 @@ public:
       // one extra element because Pascal arrays are 1-based
       amount = (numElem + 1) * elemSize;
     }
-    if (trace_mem->IsEnabled("libtexmf", MiKTeX::Trace::TraceLevel::Trace))
-    {
-      trace_mem->WriteFormattedLine("libtexmf", MIKTEXTEXT("Reallocate %s: p == %p, elementSize == %u, nElements == %u, bytes == %u"), arrayName.empty() ? "array" : arrayName.c_str(), ptr, (unsigned)elemSize, (unsigned)numElem, (unsigned)amount);
-    }
+    trace_mem->WriteLine("libtexmf", "reallocate " + arrayName + ": ptr == " + std::string(ptr == nullptr ? "nullptr" : "...") + ", elementSize == " + std::to_string(elemSize) + ", nElements == " + std::to_string(numElem));
     ptr = MiKTeX::Debug::Realloc(ptr, amount, sourceLocation);
-    if (trace_mem->IsEnabled("libtexmf", MiKTeX::Trace::TraceLevel::Trace))
-    {
-      trace_mem->WriteFormattedLine("libtexmf", MIKTEXTEXT("Reallocate: return %p"), ptr);
-    }
     return ptr;
   }
 };
