@@ -1,6 +1,6 @@
 /* miktex/TeXAndFriends/WebAppInputLine.h:              -*- C++ -*-
 
-   Copyright (C) 1996-2019 Christian Schenk
+   Copyright (C) 1996-2020 Christian Schenk
 
    This file is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published
@@ -60,9 +60,9 @@ public:
 public:
   virtual C4P::C4P_signed32 bufsize() = 0;
 public:
-  virtual char* nameoffile() = 0;
+  virtual char*& nameoffile() = 0;
 public:
-  virtual C4P::C4P_signed16& namelength() = 0;
+  virtual C4P::C4P_signed32& namelength() = 0;
 public:
   virtual char* buffer() = 0;
 public:
@@ -145,11 +145,11 @@ public:
   MiKTeX::Core::PathName GetNameOfFile() const
   {
     IInputOutput* inputOutput = GetInputOutput();
-    return inputOutput->nameoffile();
+    return MiKTeX::Core::PathName(inputOutput->nameoffile());
   }
 
 public:
-  void SetNameOfFile(const MiKTeX::Core::PathName& fileName) const
+  virtual void SetNameOfFile(const MiKTeX::Core::PathName& fileName)
   {
     IInputOutput* inputOutput = GetInputOutput();
     MiKTeX::Util::StringUtil::CopyString(inputOutput->nameoffile(), MiKTeX::Core::BufferSizes::MaxPath + 1, fileName.GetData());
