@@ -19,6 +19,9 @@
    Software Foundation, 59 Temple Place - Suite 330, Boston, MA
    02111-1307, USA. */
 
+#include <fmt/format.h>
+#include <fmt/ostream.h>
+
 #include <miktex/Core/ConfigNames>
 
 #include "internal.h"
@@ -294,42 +297,59 @@ bool WebAppInputLine::OpenOutputFile(C4P::FileRoot& f, const PathName& fileName,
   if (pimpl->enablePipes && lpszPath[0] == '|')
   {
     string command = lpszPath + 1;
+#if defined(MIKTEX_WINDOWS)
+    std::replace(command.begin(), command.end(), '\'', '"');
+#endif
     Session::ExamineCommandLineResult examineResult;
     string examinedCommand;
-    string toBeExecuted;
-    tie(examineResult, examinedCommand, toBeExecuted) = session->ExamineCommandLine(command);
+    string safeCommandLine;
+    tie(examineResult, examinedCommand, safeCommandLine) = session->ExamineCommandLine(command);
     if (examineResult == Session::ExamineCommandLineResult::SyntaxError)
     {
-      LogError("command line syntax error: " + command);
+      LogError(fmt::format("syntax error: {0}", command));
       return false;
     }
     if (examineResult != Session::ExamineCommandLineResult::ProbablySafe && examineResult != Session::ExamineCommandLineResult::MaybeSafe)
     {
-      LogError("command is unsafe: " + command);
+      LogError(fmt::format("command is unsafe: {0}", command));
       return false;
     }
+    string toBeExecuted;
     switch (pimpl->shellCommandMode)
     {
     case ShellCommandMode::Unrestricted:
+      if (session->RunningAsAdministrator() && !session->GetConfigValue(MIKTEX_CONFIG_SECTION_CORE, MIKTEX_CONFIG_VALUE_ALLOW_UNRESTRICTED_SUPER_USER).GetBool())
+      {
+        LogError(fmt::format("not allowed with elevated privileges: {0}", command));
+        return false;
+      }
       toBeExecuted = command;
       break;
     case ShellCommandMode::Forbidden:
-      LogError("command not executed: " + command);
+      LogError(fmt::format("command not executed: {0}", command));
       return false;
     case ShellCommandMode::Query:
       // TODO
     case ShellCommandMode::Restricted:
       if (examineResult != Session::ExamineCommandLineResult::ProbablySafe)
       {
-        LogError("command not allowed: " + command);
+        LogError(fmt::format("command not allowed: {0}", command));
         return false;
       }
+      toBeExecuted = safeCommandLine;
       break;
     default:
       MIKTEX_UNEXPECTED();
     }
-    LogInfo("executing output pipe: " + toBeExecuted);
-    file = session->OpenFile(toBeExecuted, FileMode::Command, FileAccess::Write, false);
+    if (examineResult == Session::ExamineCommandLineResult::ProbablySafe)
+    {
+      LogInfo(fmt::format("executing restricted output pipe: {0}", toBeExecuted));
+    }
+    else
+    {
+      LogWarn(fmt::format("executing unrestricted output pipe: {0}", toBeExecuted));
+    }
+    file = session->OpenFile(PathName(toBeExecuted), FileMode::Command, FileAccess::Write, false);
   }
   else
   {
@@ -341,19 +361,19 @@ bool WebAppInputLine::OpenOutputFile(C4P::FileRoot& f, const PathName& fileName,
       lpszPath = unmangled.GetData();
     }
 #endif
-    bool isAuxFile = !IsOutputFile(lpszPath);
+    bool isAuxFile = !IsOutputFile(PathName(lpszPath));
     PathName path;
     if (isAuxFile && !pimpl->auxDirectory.Empty())
     {
-      path = pimpl->auxDirectory / lpszPath;
+      path = pimpl->auxDirectory / PathName(lpszPath);
       lpszPath = path.GetData();
     }
     else if (!pimpl->outputDirectory.Empty())
     {
-      path = pimpl->outputDirectory / lpszPath;
+      path = pimpl->outputDirectory / PathName(lpszPath);
       lpszPath = path.GetData();
     }
-    file = session->TryOpenFile(lpszPath, FileMode::Create, FileAccess::Write, false);
+    file = session->TryOpenFile(PathName(lpszPath), FileMode::Create, FileAccess::Write, false);
     if (file != nullptr)
     {
       outPath = lpszPath;
@@ -387,6 +407,9 @@ bool WebAppInputLine::OpenInputFile(FILE** ppFile, const PathName& fileName)
   if (pimpl->enablePipes && lpszFileName[0] == '|')
   {
     string command = lpszFileName + 1;
+#if defined(MIKTEX_WINDOWS)
+    std::replace(command.begin(), command.end(), '\'', '"');
+#endif
     Session::ExamineCommandLineResult examineResult;
     string examinedCommand;
     string toBeExecuted;
@@ -422,7 +445,7 @@ bool WebAppInputLine::OpenInputFile(FILE** ppFile, const PathName& fileName)
       MIKTEX_UNEXPECTED();
     }
     LogInfo("executing input pipe: " + toBeExecuted);
-    *ppFile = session->OpenFile(toBeExecuted, FileMode::Command, FileAccess::Read, false);
+    *ppFile = session->OpenFile(PathName(toBeExecuted), FileMode::Command, FileAccess::Read, false);
     pimpl->foundFile.Clear();
     pimpl->foundFileFq.Clear();
   }
@@ -443,10 +466,10 @@ bool WebAppInputLine::OpenInputFile(FILE** ppFile, const PathName& fileName)
     }
 
     pimpl->foundFileFq = pimpl->foundFile;
-    pimpl->foundFileFq.MakeAbsolute();
+    pimpl->foundFileFq.MakeFullyQualified();
 
 #if 1 // 2015-01-15
-    if (pimpl->foundFile[0] == '.' && PathName::IsDirectoryDelimiter(pimpl->foundFile[1]))
+    if (pimpl->foundFile[0] == '.' && PathNameUtil::IsDirectoryDelimiter(pimpl->foundFile[1]))
     {
       PathName temp(pimpl->foundFile.GetData() + 2);
       pimpl->foundFile = temp;
@@ -459,23 +482,23 @@ bool WebAppInputLine::OpenInputFile(FILE** ppFile, const PathName& fileName)
       {
         CommandLineBuilder cmd("zcat");
         cmd.AppendArgument(pimpl->foundFile);
-        *ppFile = session->OpenFile(cmd.ToString(), FileMode::Command, FileAccess::Read, false);
+        *ppFile = session->OpenFile(PathName(cmd.ToString()), FileMode::Command, FileAccess::Read, false);
       }
       else if (pimpl->foundFile.HasExtension(".bz2"))
       {
         CommandLineBuilder cmd("bzcat");
         cmd.AppendArgument(pimpl->foundFile);
-        *ppFile = session->OpenFile(cmd.ToString(), FileMode::Command, FileAccess::Read, false);
+        *ppFile = session->OpenFile(PathName(cmd.ToString()), FileMode::Command, FileAccess::Read, false);
       }
       else if (pimpl->foundFile.HasExtension(".xz") || pimpl->foundFile.HasExtension(".lzma"))
       {
         CommandLineBuilder cmd("xzcat");
         cmd.AppendArgument(pimpl->foundFile);
-        *ppFile = session->OpenFile(cmd.ToString(), FileMode::Command, FileAccess::Read, false);
+        *ppFile = session->OpenFile(PathName(cmd.ToString()), FileMode::Command, FileAccess::Read, false);
       }
       else
       {
-        *ppFile = session->OpenFile(pimpl->foundFile.GetData(), FileMode::Open, FileAccess::Read, false);
+        *ppFile = session->OpenFile(pimpl->foundFile, FileMode::Open, FileAccess::Read, false);
       }
     }
 #if defined(MIKTEX_WINDOWS)
@@ -593,6 +616,7 @@ void WebAppInputLine::EnableShellCommands(ShellCommandMode mode)
   {
     return;
   }
+  auto session = GetSession();
   switch (mode)
   {
   case ShellCommandMode::Forbidden:
@@ -602,6 +626,11 @@ void WebAppInputLine::EnableShellCommands(ShellCommandMode mode)
     LogInfo("allowing known shell commands");
     break;
   case ShellCommandMode::Unrestricted:
+    if (session->RunningAsAdministrator() && !session->GetConfigValue(MIKTEX_CONFIG_SECTION_CORE, MIKTEX_CONFIG_VALUE_ALLOW_UNRESTRICTED_SUPER_USER).GetBool())
+    {
+      LogError("unrestricted shell commands not allowed when running with elevated privileges");
+      return;
+    }
     LogInfo("allowing all shell commands");
     break;
   default:
