@@ -19,6 +19,9 @@
    Software Foundation, 59 Temple Place - Suite 330, Boston, MA
    02111-1307, USA. */
 
+#include <fmt/format.h>
+#include <fmt/ostream.h>
+
 #include <miktex/Core/ConfigNames>
 #include <miktex/Core/Directory>
 #include <miktex/Core/Paths>
@@ -84,8 +87,8 @@ TeXMFApp::~TeXMFApp() noexcept
 STATICFUNC(void) TraceExecutionTime(TraceStream* trace_time, clock_t clockStart)
 {
   clock_t clockSinceStart = clock() - clockStart;
-  trace_time->WriteFormattedLine("libtexmf", T_("gross execution time: %u ms"), static_cast<unsigned>(clockSinceStart));
-  cerr << StringUtil::FormatString(T_("gross execution time: %u ms\n"), static_cast<unsigned>(clockSinceStart)) << endl;
+  trace_time->WriteLine("libtexmf", fmt::format(T_("gross execution time: {0} ms"), clockSinceStart));
+  cerr << fmt::format(T_("gross execution time: {0} ms\n"), clockSinceStart) << endl;
 #if defined(MIKTEX_WINDOWS)
   HINSTANCE hinstKernel;
   hinstKernel = LoadLibraryW(L"kernel32.dll");
@@ -113,9 +116,9 @@ STATICFUNC(void) TraceExecutionTime(TraceStream* trace_time, clock_t clockStart)
   tKernel64.HighPart = ftKernel.dwHighDateTime;
   tUser = static_cast<DWORD>(tUser64.QuadPart / 10000);
   tKernel = static_cast<DWORD>(tKernel64.QuadPart / 10000);
-  trace_time->WriteFormattedLine("libtexmf", T_("user mode: %u ms, kernel mode: %u ms, total: %u"), static_cast<unsigned>(tUser), static_cast<unsigned>(tKernel), static_cast<unsigned>(tUser + tKernel));
+  trace_time->WriteLine("libtexmf", fmt::format(T_("user mode: {0} ms, kernel mode: {1} ms, total: {2}"), tUser, tKernel, tUser + tKernel));
   cerr
-    << StringUtil::FormatString(T_("user mode: %u ms, kernel mode: %u ms, total: %u"), static_cast<unsigned>(tUser), static_cast<unsigned>(tKernel), static_cast<unsigned>(tUser + tKernel))
+    << fmt::format(T_("user mode: {0} ms, kernel mode: {1} ms, total: {2}"), tUser, tKernel, tUser + tKernel)
     << endl;
 #endif // MIKTEX_WINDOWS
 }
@@ -188,7 +191,7 @@ void TeXMFApp::OnTeXMFFinishJob()
       fileName = pimpl->jobName;
     }
     shared_ptr<Session> session = GetSession();
-    session->SetRecorderPath(PathName(GetOutputDirectory(), fileName).AppendExtension(".fls"));
+    session->SetRecorderPath(PathName(GetOutputDirectory(), PathName(fileName)).AppendExtension(".fls"));
   }
   if (pimpl->timeStatistics)
   {
@@ -354,8 +357,8 @@ bool TeXMFApp::ProcessOption(int opt, const string& optArg)
 
   case OPT_AUX_DIRECTORY:
   {
-    PathName auxDirectory = optArg;
-    auxDirectory.MakeAbsolute();
+    PathName auxDirectory(optArg);
+    auxDirectory.MakeFullyQualified();
     SetAuxDirectory(auxDirectory);
     if (!Directory::Exists(auxDirectory))
     {
@@ -464,7 +467,7 @@ bool TeXMFApp::ProcessOption(int opt, const string& optArg)
       time_t creationTime;
       time_t lastAccessTime;
       time_t lastWriteTime;
-      File::GetTimes(optArg, creationTime, lastAccessTime, lastWriteTime);
+      File::GetTimes(PathName(optArg), creationTime, lastAccessTime, lastWriteTime);
       jobTime = lastWriteTime;
     }
     SetStartUpTime(jobTime, false);
@@ -494,8 +497,8 @@ bool TeXMFApp::ProcessOption(int opt, const string& optArg)
 
   case OPT_OUTPUT_DIRECTORY:
   {
-    PathName outputDirectory = optArg;
-    outputDirectory.MakeAbsolute();
+    PathName outputDirectory(optArg);
+    outputDirectory.MakeFullyQualified();
     SetOutputDirectory(outputDirectory);
     if (!Directory::Exists(outputDirectory))
     {
@@ -555,7 +558,7 @@ bool TeXMFApp::ProcessOption(int opt, const string& optArg)
     break;
 
   case OPT_TCX:
-    SetTcxFileName(optArg);
+    SetTcxFileName(PathName(optArg));
     break;
 
   case OPT_UNDUMP:
@@ -708,7 +711,7 @@ bool TeXMFApp::OpenMemoryDumpFile(const PathName& fileName_, FILE** ppFile, void
   }
 #endif
 
-  FileStream stream(session->OpenFile(path.GetData(), FileMode::Open, FileAccess::Read, false));
+  FileStream stream(session->OpenFile(path, FileMode::Open, FileAccess::Read, false));
 
   if (pBuf != nullptr)
   {
@@ -747,7 +750,7 @@ void TeXMFApp::ProcessCommandLineOptions()
     && GetArgV()[1][0] != '\\'
     && (pimpl->memoryDumpFileName.empty() || GetTcxFileName().Empty()))
   {
-    CheckFirstLine(GetArgV()[1]);
+    CheckFirstLine(PathName(GetArgV()[1]));
   }
 }
 
@@ -802,21 +805,16 @@ PathName TeXMFApp::GetDefaultMemoryDumpFileName() const
   return name;
 }
 
-bool IsFileNameArgument(const char* lpszArg)
+bool IsFileNameArgument(const char* arg)
 {
-  for (size_t l = 0; lpszArg[l] != 0; ++l)
+  for (size_t l = 0; arg[l] != 0; ++l)
   {
-    if (l >= BufferSizes::MaxPath)
-    {
-      return false;
-    }
-    char ch = lpszArg[l];
+    char ch = arg[l];
     if (ch == '<'
       || ch == '>'
       || ch == '"'
       || ch == '|'
-      || ch == '*'
-      || ch == '?')
+      || ch == '*')
     {
       return false;
     }
@@ -868,7 +866,7 @@ void TeXMFApp::InitializeBuffer() const
       else
 #endif
 #if defined(MIKTEX_WINDOWS)
-        fileName = Q_(path.ToLongPathName().ToUnix());
+        fileName = Q_(path.ToUnix());
 #else
         fileName = Q_(path);
 #endif
