@@ -273,10 +273,10 @@ bool WebApp::ProcessOption(int opt, const string& optArg)
     EnableInstaller(TriState::True);
     break;
   case OPT_INCLUDE_DIRECTORY:
-    if (Directory::Exists(optArg))
+    if (Directory::Exists(PathName(optArg)))
     {
       PathName path(optArg);
-      path.MakeAbsolute();
+      path.MakeFullyQualified();
       session->AddInputDirectory(path, true);
     }
     break;
