@@ -19,10 +19,15 @@
 #endif
 
 #if !defined(FOR_SHARED) && \
-  ((defined(HAVE_LIBGL) && defined(HAVE_LIBGLUT) && defined(HAVE_LIBGLM)) || defined(HAVE_LIBOSMESA))
+  ((defined(HAVE_LIBGL) && defined(HAVE_LIBGLUT) && defined(HAVE_LIBGLM)) || \
+   defined(HAVE_LIBOSMESA))
 #define HAVE_GL
 #endif
 
+#if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDIT)
+#define HAVE_READLINE
+#endif
+
 #ifdef HAVE_PTHREAD
 #include <pthread.h>
 #endif
