@@ -570,7 +570,7 @@ void MainWindow::StartTerminal()
     bool haveOldPath = Utils::GetEnvironmentString("PATH", oldPath);
     if (haveOldPath)
     {
-      newPath += PathName::PathNameDelimiter;
+      newPath += PathNameUtil::PathNameDelimiter;
       newPath += oldPath;
     }
     Utils::SetEnvironmentString("PATH", newPath);
@@ -581,7 +581,7 @@ void MainWindow::StartTerminal()
       cmd = "cmd.exe";
     }
 #elif defined(MIKTEX_MACOS_BUNDLE)
-    cmd = session->GetMyLocation(true) / ".." / "Resources" / "Terminal";
+    cmd = session->GetMyLocation(true) / PathName("..") / PathName("Resources") / PathName("Terminal");
 #else
     const static string terminals[] = { "konsole", "gnome-terminal", "xterm" };
     for (const string& term : terminals)
@@ -671,7 +671,7 @@ void MainWindow::RestartAdminWithArguments(const vector<string>& args)
     MIKTEX_FATAL_WINDOWS_ERROR("ShellExecuteExW");
   }
 #elif defined(MIKTEX_MACOS_BUNDLE)
-  PathName console = session->GetMyLocation(true) / ".." / "Resources" / MIKTEX_CONSOLE_ADMIN_EXE;
+  PathName console = session->GetMyLocation(true) / PathName("..") / PathName("Resources") / PathName(MIKTEX_CONSOLE_ADMIN_EXE);
   vector<string> consoleArgs{ MIKTEX_CONSOLE_ADMIN_EXE };
   consoleArgs.insert(consoleArgs.end(), args.begin(), args.end());
   Process::Start(console, consoleArgs);
@@ -1654,7 +1654,7 @@ bool CheckRoot(const PathName& root)
 {
   for (const string& dir : tdsDirs)
   {
-    if (Directory::Exists(root / dir))
+    if (Directory::Exists(root / PathName(dir)))
     {
       return true;
     }
@@ -1671,7 +1671,7 @@ void MainWindow::AddRootDirectory()
     {
       return;
     }
-    PathName root = directory.toUtf8().constData();
+    PathName root(directory.toUtf8().constData());
     if (!CheckRoot(root))
     {
       if (QMessageBox::question(this, tr("MiKTeX Console"), tr("This does not look like a <a href=\"https://miktex.org/kb/tds\">TDS-compliant</a> root directory. Are you sure you want to add it?"))
@@ -1825,7 +1825,7 @@ void MainWindow::ChangeLinkTargetDirectory()
       return;
     }
     QThread* thread = new QThread;
-    ChangeLinkTargetDirectoryWorker* worker = new ChangeLinkTargetDirectoryWorker(directory.toUtf8().constData());
+    ChangeLinkTargetDirectoryWorker* worker = new ChangeLinkTargetDirectoryWorker(PathName(directory.toUtf8().constData()));
     backgroundWorkers++;
     ui->labelBackgroundTask->setText(tr("Changing link target directory..."));
     worker->moveToThread(thread);
@@ -2381,7 +2381,7 @@ void MainWindow::on_pushButtonShowLogDirectory_clicked()
 
 PathName MainWindow::GetReportFileName()
 {
-  return session->GetSpecialPath(SpecialPath::LogDirectory) / "miktex-report.txt";
+  return session->GetSpecialPath(SpecialPath::LogDirectory) / PathName("miktex-report.txt");
 }
 
 void MainWindow::CreateReport()
@@ -2713,7 +2713,7 @@ void MainWindow::Uninstall()
 
 void MainWindow::ReadSettings()
 {
-  PathName consoleIni = session->GetSpecialPath(SpecialPath::ConfigRoot) / MIKTEX_PATH_MIKTEX_CONFIG_DIR / "console.ini";
+  PathName consoleIni = session->GetSpecialPath(SpecialPath::ConfigRoot) / PathName(MIKTEX_PATH_MIKTEX_CONFIG_DIR) / PathName("console.ini");
   if (!File::Exists(consoleIni))
   {
     return;
@@ -2732,7 +2732,7 @@ void MainWindow::WriteSettings()
 {
   unique_ptr<Cfg> settings = Cfg::Create();
   settings->PutValue("MainWindow", "geometry", saveGeometry().toHex().constData());
-  settings->Write(session->GetSpecialPath(SpecialPath::ConfigRoot) / MIKTEX_PATH_MIKTEX_CONFIG_DIR / "console.ini");
+  settings->Write(session->GetSpecialPath(SpecialPath::ConfigRoot) / PathName(MIKTEX_PATH_MIKTEX_CONFIG_DIR) / PathName("console.ini"));
 }
 
 void MainWindow::Restart()
