@@ -317,7 +317,7 @@ private:
   vector<TraceCallback::TraceMessage> pendingTraceMessages;
 
 public:
-  void MIKTEXTHISCALL Trace(const TraceCallback::TraceMessage& traceMessage) override
+  bool MIKTEXTHISCALL Trace(const TraceCallback::TraceMessage& traceMessage) override
   {
     if (!isLog4cxxConfigured)
     {
@@ -326,10 +326,11 @@ public:
         pendingTraceMessages.clear();
       }
       pendingTraceMessages.push_back(traceMessage);
-      return;
+      return true;
     }
     FlushPendingTraceMessages();
     TraceInternal(traceMessage);
+    return true;
   }
 
 private:
@@ -1448,7 +1449,7 @@ void Application::RestartWindowed()
   // locate miktex-console
   PathName miktexConsole;
 #if defined(MIKTEX_MACOS_BUNDLE)
-  miktexConsole = session->GetSpecialPath(SpecialPath::MacOsDirectory) / MIKTEX_MACOS_BUNDLE_NAME;
+  miktexConsole = session->GetSpecialPath(SpecialPath::MacOsDirectory) / PathName(MIKTEX_MACOS_BUNDLE_NAME);
 #else
   if (!session->FindFile(miktexConsoleName, FileType::EXE, miktexConsole))
   {
@@ -1472,7 +1473,7 @@ void ReadNames(const PathName& path, vector<string>& list)
       if (name[0] == '@')
       {
         // RECURSION
-        ReadNames(&name[1], list);
+        ReadNames(PathName(&name[1]), list);
       }
       else
       {
@@ -1487,7 +1488,7 @@ void ParseList(const string& s, vector<string>& list)
 {
   if (s.length() > 0 && s[0] == '@')
   {
-    ReadNames(&s[1], list);
+    ReadNames(PathName(&s[1]), list);
   }
   else
   {
@@ -1616,7 +1617,7 @@ void Application::Main(int argc, const char** argv)
       // TODO
       Warn(T_("Option --install-some is deprecated"));
 #endif
-      ReadNames(optArg, toBeInstalled);
+      ReadNames(PathName(optArg), toBeInstalled);
       break;
     case OPT_LIST:
       optList = true;
@@ -1805,7 +1806,7 @@ void Application::Main(int argc, const char** argv)
         Error(T_("Already updating all packages."));
       }
       optUpdate = true;
-      ReadNames(optArg, updates);
+      ReadNames(PathName(optArg), updates);
       break;
     case OPT_UPGRADE:
       optUpgrade = true;
@@ -1899,7 +1900,7 @@ void Application::Main(int argc, const char** argv)
     {
       invokerName = "unknown process";
     }
-    LOG4CXX_INFO(logger, "this is " << Utils::MakeProgramVersionString("mpmcli", MIKTEX_COMPONENT_VERSION_STR));
+    LOG4CXX_INFO(logger, "this is " << Utils::MakeProgramVersionString("mpmcli", VersionNumber(MIKTEX_COMPONENT_VERSION_STR)));
     LOG4CXX_INFO(logger, "this process (" << thisProcess->GetSystemId() << ") started by '" << invokerName << "' with command line: " << CommandLineBuilder(argc, argv));
   }
 
