@@ -206,7 +206,7 @@ void ParseYapCommandLine(const char* lpszArguments, YapCommandLineInfo& cmdInfo)
     {
       // we have a file name argument; append .dvi to the file
       // name, if necessary
-      PathName pathFileName = arg;
+      PathName pathFileName(arg);
       if (!pathFileName.HasExtension(".dvi"))
       {
         pathFileName.Append(".dvi", false);
@@ -250,7 +250,7 @@ namespace
   bool isLog4cxxConfigured = false;
 }
 
-void YapApplication::Trace(const TraceCallback::TraceMessage& traceMessage)
+bool YapApplication::Trace(const TraceCallback::TraceMessage& traceMessage)
 {
   if (!isLog4cxxConfigured)
   {
@@ -259,10 +259,11 @@ void YapApplication::Trace(const TraceCallback::TraceMessage& traceMessage)
       pendingTraceMessages.clear();
     }
     pendingTraceMessages.push_back(traceMessage);
-    return;
+    return true;
   }
   FlushPendingTraceMessages();
   TraceInternal(traceMessage);
+  return true;
 }
 
 void YapApplication::TraceInternal(const TraceCallback::TraceMessage& traceMessage)
@@ -402,7 +403,7 @@ BOOL YapApplication::InitInstance()
       {
         invokerName = "unknown process";
       }
-      LOG4CXX_INFO(logger, "this is " << Utils::MakeProgramVersionString("Yap", MIKTEX_COMPONENT_VERSION_STR));
+      LOG4CXX_INFO(logger, "this is " << Utils::MakeProgramVersionString("Yap", VersionNumber(MIKTEX_COMPONENT_VERSION_STR)));
       LOG4CXX_INFO(logger, "this process (" << thisProcess->GetSystemId() << ") started by '" << invokerName << "' with command line: " << TU_(m_lpCmdLine));
     }
 
@@ -698,7 +699,7 @@ bool YapApplication::ActivateFirstInstance(const YapCommandLineInfo& cmdInfo)
 
   // open the DVI document via DDE
   PathName path(cmdInfo.m_strFileName);
-  path.MakeAbsolute();
+  path.MakeFullyQualified();
 
   CStringA ddeCommand;
   ddeCommand.Format("[open(\"%s\")]", path.GetData());
@@ -959,7 +960,7 @@ void StartEditor(const char* lpszFileName, const char* lpszDocDir, int line)
 
   // find the source file
   PathName path;
-  if (Utils::IsAbsolutePath(lpszFileName) || lpszDocDir == nullptr)
+  if (PathNameUtil::IsAbsolutePath(lpszFileName) || lpszDocDir == nullptr)
   {
     path = lpszFileName;
   }
@@ -1078,9 +1079,6 @@ CDocument* YapApplication::OpenDocumentFile(LPCTSTR lpszFileName)
   try
   {
     PathName pathShort(lpszFileName);
-#if defined(REMOVE_BLANKS_FROM_DOCUMENT_FILENAMES)
-    Utils::RemoveBlanksFromPathName(pathShort);
-#endif
     return CWinApp::OpenDocumentFile(UT_(pathShort.GetData()));
   }
   catch (const MiKTeXException& e)
