@@ -1,6 +1,6 @@
 /* epstopdf.cpp: epstopdf
 
-   Copyright (C) 2000-2019 Christian Schenk
+   Copyright (C) 2000-2020 Christian Schenk
    Copyright (C) 1998-2001 by Sebastian Rahtz et al.
 
    This file is part of EPStoPDF.
@@ -24,15 +24,19 @@
 
 #include <cassert>
 #include <cmath>
-#include <cstdarg>
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
 
 #include <iostream>
+#include <map>
 #include <memory>
+#include <regex>
 #include <string>
 
+#include <fmt/format.h>
+#include <fmt/ostream.h>
+
 #include <miktex/App/Application>
 #include <miktex/Core/BufferSizes>
 #include <miktex/Core/CommandLineBuilder>
@@ -65,24 +69,9 @@ using namespace std::string_literals;
 #define T_(x) MIKTEXTEXT(x)
 #define Q_(x) MiKTeX::Core::Quoter<char>(x).GetData()
 
-#define VA_START(arglist, format   )            \
-va_start(arglist, format);                      \
-try                                             \
-{
-
-#define VA_END(arglist)                         \
-}                                               \
-catch(...)                                      \
-{                                               \
-  va_end(arglist);                              \
-  throw;                                        \
-}                                               \
-va_end(arglist);
-
-vector<string> DEFAULT_TRACE_STREAMS = {
-  MIKTEX_TRACE_ERROR,
-  MIKTEX_TRACE_PROCESS,
-  PROGRAM_NAME
+vector<string> DEFAULT_TRACE_OPTIONS = {
+  TraceStream::MakeOption("", "", TraceLevel::Info),
+  TraceStream::MakeOption(PROGRAM_NAME, "", TraceLevel::Trace),
 };
 
 class EpsToPdfApp :
@@ -98,25 +87,31 @@ public:
   void Run(int argc, const char** argv);
 
 private:
-  void MyTrace(const char* format, ...);
+  void MyTrace(const string& line);
 
 private:
-  void Verbose(const char* format, ...);
+  void Verbose(const string& line);
 
 private:
-  void PrintOnly(const char* format, ...);
+  void PrintOnly(const string& line);
 
 private:
-  void Warning(const char* format, ...);
+  void Warning(const string& line);
 
 private:
   bool GetLine(string& line);
 
 private:
-  void PutFormattedLine(const char* format, ...);
+  void PutLine(const string& line);
 
 private:
-  void PutLine(const string& line);
+  void ExamineLine(const string& line);
+
+private:
+  void EnsureFontIsAvailable(const string& fontName);
+
+private:
+  void ReadFontMapFile(const PathName& path);
 
 private:
   void CorrectBoundingBox(double llx, double lly, double urx, double ury);
@@ -164,7 +159,7 @@ private:
   FileStream inStream;
 
 private:
-  long stopReadingAt;
+  long stopReadingAt = 0;
 
 private:
   FileStream outStream;
@@ -172,6 +167,15 @@ private:
 private:
   bool boundingBoxCorrected = false;
 
+private:
+  map<string, string> fontMap;
+
+private:
+  const regex findfont_regex{ "/([A-Za-z0-9_\\-]+)\\s+findfont" };
+
+private:
+  const regex fontmap_regex{ "/([A-Za-z0-9_\\-]+)\\s+([\\(/A-Za-z0-9_\\.\\)\\-]+)\\s*;" };
+
 private:
   unique_ptr<TraceStream> traceStream;
 
@@ -420,44 +424,32 @@ struct poptOption EpsToPdfApp::aoption[] = {
   POPT_TABLEEND
 };
 
-void EpsToPdfApp::MyTrace(const char* format, ...)
+void EpsToPdfApp::MyTrace(const string& line)
 {
-  va_list arglist;
-  VA_START(arglist, format);
-  traceStream->WriteLine(PROGRAM_NAME, StringUtil::FormatStringVA(format, arglist).c_str());
-  VA_END(arglist);
+  traceStream->WriteLine(PROGRAM_NAME, line);
 }
 
-void EpsToPdfApp::Verbose(const char* format, ...)
+void EpsToPdfApp::Verbose(const string& line)
 {
   if (!verbose || printOnly)
   {
     return;
   }
-  va_list arglist;
-  VA_START(arglist, format);
-  cout << StringUtil::FormatStringVA(format, arglist) << endl;
-  VA_END(arglist);
+  cout << line << endl;
 }
 
-void EpsToPdfApp::PrintOnly(const char* format, ...)
+void EpsToPdfApp::PrintOnly(const string& line)
 {
   if (!printOnly)
   {
     return;
   }
-  va_list arglist;
-  VA_START(arglist, format);
-  cout << StringUtil::FormatStringVA(format, arglist) << endl;
-  VA_END(arglist);
+  cout << line << endl;
 }
 
-void EpsToPdfApp::Warning(const char* format, ...)
+void EpsToPdfApp::Warning(const string& line)
 {
-  va_list arglist;
-  VA_START(arglist, format);
-  cerr << T_("warning") << ": " << StringUtil::FormatStringVA(format, arglist) << endl;
-  VA_END(arglist);
+  cerr << T_("warning") << ": " << line << endl;
 }
 
 bool EpsToPdfApp::GetLine(string& line)
@@ -470,7 +462,7 @@ bool EpsToPdfApp::GetLine(string& line)
       return false;
     }
   }
-  bool done = Utils::ReadUntilDelim(line, '\n', inStream.GetFile());
+  bool done = Utils::ReadLine(line, inStream.GetFile(), true);
   if (done)
   {
     string::size_type l = line.length();
@@ -487,18 +479,6 @@ bool EpsToPdfApp::GetLine(string& line)
   return done;
 }
 
-void EpsToPdfApp::PutFormattedLine(const char* format, ...)
-{
-  if (!printOnly)
-  {
-    va_list marker;
-    VA_START(marker, format);
-    vfprintf(outStream.GetFile(), format, marker);
-    VA_END(marker);
-    fputc('\n', outStream.GetFile());
-  }
-}
-
 void EpsToPdfApp::PutLine(const string& line)
 {
   if (!printOnly)
@@ -507,9 +487,70 @@ void EpsToPdfApp::PutLine(const string& line)
   }
 }
 
+void EpsToPdfApp::ExamineLine(const string& line)
+{
+  smatch m;
+  string s = line;
+  while (regex_search(s, m, findfont_regex))
+  {
+    EnsureFontIsAvailable(m[1]);
+    s = m.suffix();
+  }
+}
+
+void EpsToPdfApp::EnsureFontIsAvailable(const string& fontName)
+{
+  map<string, string>::const_iterator it = fontMap.find(fontName);
+  if (it == fontMap.end())
+  {
+    Warning(fmt::format(T_("required font {0} not found in font map"), fontName));
+    return;
+  }
+  const string& s = it->second;
+  MIKTEX_ASSERT(!s.empty());
+  if (s[0] == '(' && s[s.length() - 1] == ')')
+  {
+    string fileName = s.substr(1, s.length() - 2);
+    PathName result;
+    if (session->FindFile(fileName, "%R/fonts//", result))
+    {
+      MyTrace(fmt::format(T_("found required font {0}: {1}"), fontName, result));
+    }
+    else
+    {
+      Warning(fmt::format(T_("required font not found: {0} ({1})"), fontName, fileName));
+    }
+  }
+  else if (s[0] == '/')
+  {
+    MIKTEX_ASSERT(s.length() > 1);
+    EnsureFontIsAvailable(s.substr(1));
+  }
+  else
+  {
+    MIKTEX_UNEXPECTED();
+  }
+}
+
+void EpsToPdfApp::ReadFontMapFile(const PathName& path)
+{
+  std::ifstream reader = File::CreateInputStream(path);
+  size_t n = 0;
+  for (string line; std::getline(reader, line); )
+  {
+    smatch m;
+    if (regex_match(line, m, fontmap_regex))
+    {
+      fontMap[m[1]] = m[2];
+      n++;
+    }
+  }
+  MyTrace(fmt::format(T_("read font map file {0} with {1} matches"), Q_(path), n));
+}
+
 void EpsToPdfApp::CorrectBoundingBox(double llx, double lly, double urx, double ury)
 {
-  MyTrace(T_("Old BoundingBox: %f %f %f %f"), llx, lly, urx, ury);
+  MyTrace(fmt::format(T_("Old BoundingBox: {0:.2f} {1:.2f} {2:.2f} {3:.2f}"), llx, lly, urx, ury));
   llx -= enlarge;
   lly -= enlarge;
   urx += enlarge;
@@ -518,11 +559,11 @@ void EpsToPdfApp::CorrectBoundingBox(double llx, double lly, double urx, double
   int height = static_cast<int>(ceil(ury - lly));
   int xoffset = static_cast<int>(-llx);
   int yoffset = static_cast<int>(-lly);
-  MyTrace(T_("New BoundingBox: 0 0 %d %d"), width, height);
-  MyTrace(T_("Offset: %d %d"), xoffset, yoffset);
-  PutFormattedLine("%%%%BoundingBox: 0 0 %d %d", width, height);
-  PutFormattedLine("<< /PageSize [%d %d] >> setpagedevice", width, height);
-  PutFormattedLine("gsave %d %d translate", xoffset, yoffset);
+  MyTrace(fmt::format(T_("New BoundingBox: 0 0 {0} {1}"), width, height));
+  MyTrace(fmt::format(T_("Offset: {0} {1}"), xoffset, yoffset));
+  PutLine(fmt::format("%%%%BoundingBox: 0 0 {0} {1}", width, height));
+  PutLine(fmt::format("<< /PageSize [{0} {1}] >> setpagedevice", width, height));
+  PutLine(fmt::format("gsave {0} {1} translate", xoffset, yoffset));
 }
 
 bool EpsToPdfApp::BoundingBoxWithValues(const string& line, double& llx, double& lly, double& urx, double& ury)
@@ -578,14 +619,14 @@ void EpsToPdfApp::ScanHeader()
 
     if (BoundingBoxWithAtEnd(line))
     {
-      MyTrace(T_("%s (atend)"), boundingBoxName.c_str());
+      MyTrace(fmt::format(T_("{0} (atend)"), boundingBoxName));
       if (runAsFilter)
       {
         Warning(T_("Cannot look for BoundingBox in the trailer with option --filter."));
         break;
       }
       long pos = inStream.GetPosition();
-      MyTrace(T_("Current file position: %d"), pos);
+      MyTrace(fmt::format(T_("Current file position: {0}"), pos));
       while (GetLine(line))
       {
         if (line.compare(0, 15, "%%BeginDocument") == 0)
@@ -709,7 +750,7 @@ void EpsToPdfApp::PrepareOutput(bool runAsFilter, bool runGhostscript, const Pat
     gsOptions.push_back("-");
     gsOptions.push_back("-c");
     gsOptions.push_back("quit");
-    PrintOnly("%s\n", CommandLineBuilder(gsOptions).ToString().c_str());
+    PrintOnly(CommandLineBuilder(gsOptions).ToString());
     if (!printOnly)
     {
       ProcessStartInfo processStartInfo;
@@ -770,7 +811,7 @@ void EpsToPdfApp::Run(int argc, const char** argv)
       doCompress = true;
       break;
     case OPT_DEBUG:
-      initInfo.SetTraceFlags(StringUtil::Flatten(DEFAULT_TRACE_STREAMS, ','));
+      initInfo.SetTraceFlags(StringUtil::Flatten(DEFAULT_TRACE_OPTIONS, ','));
       break;
     case OPT_ENLARGE:
       enlarge = std::stoi(optArg);
@@ -826,7 +867,7 @@ void EpsToPdfApp::Run(int argc, const char** argv)
     case OPT_TRACE:
       if (optArg.empty())
       {
-        initInfo.SetTraceFlags(StringUtil::Flatten(DEFAULT_TRACE_STREAMS, ','));
+        initInfo.SetTraceFlags(StringUtil::Flatten(DEFAULT_TRACE_OPTIONS, ','));
       }
       else
       {
@@ -839,7 +880,7 @@ void EpsToPdfApp::Run(int argc, const char** argv)
     case OPT_VERSION:
       cout
         << Utils::MakeProgramVersionString(THE_NAME_OF_THE_GAME, VersionNumber(MIKTEX_MAJOR_VERSION, MIKTEX_MINOR_VERSION, MIKTEX_COMP_J2000_VERSION, 0)) << endl
-        << "Copyright (C) 2000-2019 Christian Schenk" << endl
+        << "Copyright (C) 2000-2020 Christian Schenk" << endl
         << "Copyright (C) 1998-2001 by Sebastian Rahtz et al." << endl
         << "This is free software; see the source for copying conditions.  There is NO" << endl
         << "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE." << endl;
@@ -862,6 +903,18 @@ void EpsToPdfApp::Run(int argc, const char** argv)
 
   vector<string> leftovers = popt.GetLeftovers();
 
+  for (const string& fileName : { "Fontmap.aliases"s, "Fontmap.MiKTeX"s })
+  {
+    vector<PathName> fontMapFiles;
+    if (session->FindFile(fileName, "%R/ghostscript/base", fontMapFiles))
+    {
+      for (const PathName& fontMapFile : fontMapFiles)
+      {
+        ReadFontMapFile(fontMapFile);
+      }
+    }
+  }
+
   if (runAsFilter)
   {
     if (!leftovers.empty())
@@ -884,7 +937,7 @@ void EpsToPdfApp::Run(int argc, const char** argv)
     {
       FatalError(T_("The input file does not exist."));
     }
-    MyTrace(T_("Input filename: %s"), inputFile.GetData());
+    MyTrace(fmt::format(T_("Input filename: {0}"), inputFile.GetData()));
   }
 
   if (runAsFilter && verbose)
@@ -939,7 +992,7 @@ void EpsToPdfApp::Run(int argc, const char** argv)
 
   if (!runAsFilter)
   {
-    Verbose(T_("Making %s from %s..."), Q_(outFile), Q_(inputFile));
+    Verbose(fmt::format(T_("Making {0} from {1}..."), Q_(outFile), Q_(inputFile)));
   }
 
   PathName gsExe;
@@ -962,6 +1015,7 @@ void EpsToPdfApp::Run(int argc, const char** argv)
 
   while (GetLine(line))
   {
+    ExamineLine(line);
     PutLine(line);
   }
 
