@@ -187,7 +187,7 @@ const struct poptOption FindTeXMF::aoption[] =
 void FindTeXMF::ShowVersion()
 {
   cout
-    << Utils::MakeProgramVersionString(TheNameOfTheGame, MIKTEX_COMPONENT_VERSION_STR) << endl
+    << Utils::MakeProgramVersionString(TheNameOfTheGame, VersionNumber(MIKTEX_COMPONENT_VERSION_STR)) << endl
     << "Copyright (C) 2001-2020 Christian Schenk" << endl
     << "This is free software; see the source for copying conditions.  There is NO" << endl
     << "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE." << endl;
@@ -202,7 +202,7 @@ void FindTeXMF::ListFileTypes()
     {
       continue;
     }
-    cout << "  " << fti.fileTypeString << " (" << StringUtil::Flatten(fti.fileNameExtensions, PathName::PathNameDelimiter) << ")" << endl;
+    cout << "  " << fti.fileTypeString << " (" << StringUtil::Flatten(fti.fileNameExtensions, PathNameUtil::PathNameDelimiter) << ")" << endl;
   }
 }
 
@@ -211,10 +211,10 @@ void FindTeXMF::PrintSearchPath(const char* lpszSearchPath)
   bool first = true;
   PathName mpmRootPath = session->GetMpmRootPath();
   size_t mpmRootPathLen = mpmRootPath.GetLength();
-  for (const string& path : StringUtil::Split(lpszSearchPath, PathName::PathNameDelimiter))
+  for (const string& path : StringUtil::Split(lpszSearchPath, PathNameUtil::PathNameDelimiter))
   {
-    if ((PathName::Compare(path, mpmRootPath, mpmRootPathLen) == 0)
-      && (path.length() == mpmRootPathLen || IsDirectoryDelimiter(path[mpmRootPathLen])))
+    if ((PathName::Compare(PathName(path), mpmRootPath, mpmRootPathLen) == 0)
+      && (path.length() == mpmRootPathLen || PathNameUtil::IsDirectoryDelimiter(path[mpmRootPathLen])))
     {
       continue;
     }
@@ -265,7 +265,7 @@ int FindTeXMF::Run(int argc, const char** argv)
 
     case OPT_FILE_TYPE:
 
-      fileType = session->DeriveFileType(optArg);
+      fileType = session->DeriveFileType(PathName(optArg));
       if (fileType == FileType::None)
       {
         FatalError(fmt::format(T_("Unknown file type: {0}."), optArg));
@@ -286,7 +286,7 @@ int FindTeXMF::Run(int argc, const char** argv)
     case OPT_SHOW_PATH:
 
     {
-      FileType filetype = session->DeriveFileType(optArg);
+      FileType filetype = session->DeriveFileType(PathName(optArg));
       if (filetype == FileType::None)
       {
         FatalError(fmt::format(T_("Unknown file type: {0}."), optArg));
@@ -349,7 +349,7 @@ int FindTeXMF::Run(int argc, const char** argv)
     FileType filetype = fileType;
     if (filetype == FileType::None)
     {
-      filetype = session->DeriveFileType(fileName);
+      filetype = session->DeriveFileType(PathName(fileName));
       if (filetype == FileType::None)
       {
         filetype = FileType::TEX;
