@@ -388,7 +388,7 @@ private:
   }
   
 public:
-  void Trace(const TraceCallback::TraceMessage& traceMessage) override
+  bool Trace(const TraceCallback::TraceMessage& traceMessage) override
   {
     if (!isLog4cxxConfigured)
     {
@@ -396,10 +396,11 @@ public:
       fprintf(stderr, "%s\n", traceMessage.message.c_str());
 #endif
       PushTraceMessage(traceMessage);
-      return;
+      return true;
     }
     FlushPendingTraceMessages();
     LogTraceMessage(traceMessage);
+    return true;
   }
 
 private:
@@ -734,7 +735,7 @@ void IniTeXMFApp::Init(int argc, const char* argv[])
   {
     invokerName = "unknown process";
   }
-  LOG4CXX_INFO(logger, "this is " << Utils::MakeProgramVersionString(TheNameOfTheGame, MIKTEX_COMPONENT_VERSION_STR));
+  LOG4CXX_INFO(logger, "this is " << Utils::MakeProgramVersionString(TheNameOfTheGame, VersionNumber(MIKTEX_COMPONENT_VERSION_STR)));
   LOG4CXX_INFO(logger, "this process (" << thisProcess->GetSystemId() << ") started by '" << invokerName << "' with command line: " << CommandLineBuilder(argc, argv));
   FlushPendingTraceMessages();
   if (session->IsAdminMode())
@@ -746,8 +747,8 @@ void IniTeXMFApp::Init(int argc, const char* argv[])
     Verbose(T_("Operating on the private (per-user) MiKTeX setup"));
   }
   PathName myName = PathName(argv[0]).GetFileNameWithoutExtension();
-  isMktexlsrMode = myName == "mktexlsr" || myName == "texhash";
-  isTexlinksMode = myName == "texlinks";
+  isMktexlsrMode = myName == PathName("mktexlsr") || myName == PathName("texhash");
+  isTexlinksMode = myName == PathName("texlinks");
   session->SetFindFileCallback(this);
 }
 
@@ -1201,21 +1202,21 @@ void IniTeXMFApp::ManageLink(const FileLink& fileLink, bool supportsHardLinks, b
 #if defined(MIKTEX_UNIX)
     fileExistsOptions += FileExistsOption::SymbolicLink;
 #endif
-    if (File::Exists(linkName, fileExistsOptions))
+    if (File::Exists(PathName(linkName), fileExistsOptions))
     {
-      if (!isRemoveRequested && (!allowOverwrite || (linkType == LinkType::Copy && File::Equals(fileLink.target, linkName))))
+      if (!isRemoveRequested && (!allowOverwrite || (linkType == LinkType::Copy && File::Equals(PathName(fileLink.target), PathName(linkName)))))
       {
         continue;
       }
 #if defined(MIKTEX_UNIX)
-      if (File::IsSymbolicLink(linkName))
+      if (File::IsSymbolicLink(PathName(linkName)))
       {
-        PathName linkTarget = File::ReadSymbolicLink(linkName);
+        PathName linkTarget = File::ReadSymbolicLink(PathName(linkName));
 	string linkTargetFileName = linkTarget.GetFileName().ToString();
-        bool isMiKTeXSymlinked = linkTargetFileName.find(MIKTEX_PREFIX) == 0 || linkTargetFileName == PathName(fileLink.target).GetFileName();
+        bool isMiKTeXSymlinked = linkTargetFileName.find(MIKTEX_PREFIX) == 0 || PathName(linkTargetFileName) == PathName(fileLink.target).GetFileName();
         if (!isMiKTeXSymlinked)
         {
-          if (File::Exists(linkTarget))
+          if (File::Exists(PathName(linkTarget)))
           {
             LOG4CXX_WARN(logger, Q_(linkName) << " already symlinked to " << Q_(linkTarget));
             continue;
@@ -1230,7 +1231,7 @@ void IniTeXMFApp::ManageLink(const FileLink& fileLink, bool supportsHardLinks, b
       PrintOnly(fmt::format("rm {}", Q_(linkName)));
       if (!printOnly)
       {
-        File::Delete(linkName, { FileDeleteOption::TryHard, FileDeleteOption::UpdateFndb });
+        File::Delete(PathName(linkName), { FileDeleteOption::TryHard, FileDeleteOption::UpdateFndb });
       }
     }
     if (isRemoveRequested)
@@ -1255,7 +1256,7 @@ void IniTeXMFApp::ManageLink(const FileLink& fileLink, bool supportsHardLinks, b
         PrintOnly(fmt::format("ln -s {} {}", Q_(linkName), Q_(target)));
         if (!printOnly)
         {
-          File::CreateLink(target, linkName, { CreateLinkOption::UpdateFndb, CreateLinkOption::Symbolic });
+          File::CreateLink(PathName(target), PathName(linkName), { CreateLinkOption::UpdateFndb, CreateLinkOption::Symbolic });
         }
         break;
       }
@@ -1263,14 +1264,14 @@ void IniTeXMFApp::ManageLink(const FileLink& fileLink, bool supportsHardLinks, b
       PrintOnly(fmt::format("ln {} {}", Q_(fileLink.target), Q_(linkName)));
       if (!printOnly)
       {
-        File::CreateLink(fileLink.target, linkName, { CreateLinkOption::UpdateFndb });
+        File::CreateLink(PathName(fileLink.target), PathName(linkName), { CreateLinkOption::UpdateFndb });
       }
       break;
     case LinkType::Copy:
       PrintOnly(fmt::format("cp {} {}", Q_(fileLink.target), Q_(linkName)));
       if (!printOnly)
       {
-        File::Copy(fileLink.target, linkName, { FileCopyOption::UpdateFndb });
+        File::Copy(PathName(fileLink.target), PathName(linkName), { FileCopyOption::UpdateFndb });
       }
       break;
     default:
@@ -1449,7 +1450,6 @@ vector<FileLink> miktexFileLinks =
   { MIKTEX_MPOST_EXE, { "dvitomp", "mpost" } },
   { MIKTEX_ODVICOPY_EXE, { "odvicopy" } },
   { MIKTEX_OFM2OPL_EXE, { "ofm2opl" } },
-  { MIKTEX_OMEGA_EXE, { "omega" } },
   { MIKTEX_OPL2OFM_EXE, { "opl2ofm" } },
   { MIKTEX_OTP2OCP_EXE, { "otp2ocp" } },
   { MIKTEX_OUTOCP_EXE, { "outocp" } },
@@ -1555,6 +1555,9 @@ vector<FileLink> miktexFileLinks =
   { MIKTEX_PREFIX "t1testpage" MIKTEX_EXE_FILE_SUFFIX, { "t1testpage" } },
   { MIKTEX_PREFIX "ttftotype42" MIKTEX_EXE_FILE_SUFFIX, { "ttftotype42" } },
 #endif
+#if defined(WITH_OMEGA)
+  { MIKTEX_OMEGA_EXE, { "omega" } },
+#endif
 #if defined(MIKTEX_WINDOWS)
   { MIKTEX_PREFIX "zip" MIKTEX_EXE_FILE_SUFFIX, {"zip"} },
 #endif
@@ -1583,19 +1586,19 @@ vector<FileLink> IniTeXMFApp::CollectLinks(LinkCategoryOptions linkCategories)
     vector<FileLink> links = miktexFileLinks;
     links.insert(links.end(), lua52texLinks.begin(), lua52texLinks.end());
 #if defined(MIKTEX_MACOS_BUNDLE)
-    PathName console(session->GetSpecialPath(SpecialPath::MacOsDirectory) / MIKTEX_MACOS_BUNDLE_NAME);
+    PathName console(session->GetSpecialPath(SpecialPath::MacOsDirectory) / PathName(MIKTEX_MACOS_BUNDLE_NAME));
     links.push_back(FileLink(console.ToString(), { MIKTEX_CONSOLE_EXE }, LinkType::Symbolic));
 #endif
     for (const FileLink& fileLink : links)
     {
       PathName targetPath;
-      if (Utils::IsAbsolutePath(fileLink.target))
+      if (PathNameUtil::IsAbsolutePath(fileLink.target))
       {
         targetPath = fileLink.target;
       }
       else
       {
-        targetPath = pathBinDir / fileLink.target;
+        targetPath = pathBinDir / PathName(fileLink.target);
       }
       string extension = targetPath.GetExtension();
       if (File::Exists(targetPath))
@@ -1603,7 +1606,7 @@ vector<FileLink> IniTeXMFApp::CollectLinks(LinkCategoryOptions linkCategories)
         vector<string> linkNames;
         for (const string& linkName : fileLink.linkNames)
         {
-          PathName linkPath = linkTargetDirectory / linkName;
+          PathName linkPath = linkTargetDirectory / PathName(linkName);
           if (linkPath == targetPath)
           {
             continue;
@@ -1638,7 +1641,7 @@ vector<FileLink> IniTeXMFApp::CollectLinks(LinkCategoryOptions linkCategories)
         Warning(fmt::format(T_("The {0} executable could not be found."), engine));
         continue;
       }
-      PathName exePath(linkTargetDirectory, formatInfo.name);
+      PathName exePath(linkTargetDirectory, PathName(formatInfo.name));
       if (strlen(MIKTEX_EXE_FILE_SUFFIX) > 0)
       {
         exePath.AppendExtension(MIKTEX_EXE_FILE_SUFFIX);
@@ -1678,7 +1681,7 @@ vector<FileLink> IniTeXMFApp::CollectLinks(LinkCategoryOptions linkCategories)
         {
           continue;
         }
-        PathName pathExe(linkTargetDirectory, name);
+        PathName pathExe(linkTargetDirectory, PathName(name));
         if (strlen(MIKTEX_EXE_FILE_SUFFIX) > 0)
         {
           pathExe.AppendExtension(MIKTEX_EXE_FILE_SUFFIX);
@@ -1764,13 +1767,13 @@ void IniTeXMFApp::MakeLanguageDat(bool force)
     return;
   }
 
-  PathName languageDatPath = session->GetSpecialPath(SpecialPath::ConfigRoot) / MIKTEX_PATH_LANGUAGE_DAT;
+  PathName languageDatPath = session->GetSpecialPath(SpecialPath::ConfigRoot) / PathName(MIKTEX_PATH_LANGUAGE_DAT);
   ofstream languageDat = File::CreateOutputStream(languageDatPath);
 
-  PathName languageDatLuaPath = session->GetSpecialPath(SpecialPath::ConfigRoot) / MIKTEX_PATH_LANGUAGE_DAT_LUA;
+  PathName languageDatLuaPath = session->GetSpecialPath(SpecialPath::ConfigRoot) / PathName(MIKTEX_PATH_LANGUAGE_DAT_LUA);
   ofstream languageDatLua = File::CreateOutputStream(languageDatLuaPath);
 
-  PathName languageDefPath = session->GetSpecialPath(SpecialPath::ConfigRoot) / MIKTEX_PATH_LANGUAGE_DEF;
+  PathName languageDefPath = session->GetSpecialPath(SpecialPath::ConfigRoot) / PathName(MIKTEX_PATH_LANGUAGE_DEF);
   ofstream languageDef = File::CreateOutputStream(languageDefPath);
 
   languageDatLua << "return {" << "\n";
@@ -1894,7 +1897,7 @@ void IniTeXMFApp::CreateConfigFile(const string& relPath, bool edit)
   {
     PathName fileName(relPath);
     fileName.RemoveDirectorySpec();
-    if (fileName == relPath)
+    if (fileName == PathName(relPath))
     {
       configFile /= MIKTEX_PATH_MIKTEX_CONFIG_DIR;
     }
@@ -1927,7 +1930,7 @@ void IniTeXMFApp::CreateConfigFile(const string& relPath, bool edit)
       FatalError(T_("Environment variable EDITOR is not defined."));
 #endif
     }
-    Process::Start(editor, vector<string>{ editor, configFile.ToString() });
+    Process::Start(PathName(editor), vector<string>{ editor, configFile.ToString() });
   }
 }
 
@@ -2021,7 +2024,7 @@ void IniTeXMFApp::Bootstrap()
   }
   if (!neededPackages.empty())
   {
-    PathName bootstrappingDir = session->GetSpecialPath(SpecialPath::DistRoot) / MIKTEX_PATH_MIKTEX_BOOTSTRAPPING_DIR;
+    PathName bootstrappingDir = session->GetSpecialPath(SpecialPath::DistRoot) / PathName(MIKTEX_PATH_MIKTEX_BOOTSTRAPPING_DIR);
     if (Directory::Exists(bootstrappingDir))
     {
       PushTraceMessage("running MIKTEX_HOOK_BOOTSTRAPPING");
@@ -2056,7 +2059,7 @@ string kpsewhich_expand_path(const string& varname)
   return result;
 }
 
-string Concat(const initializer_list<string>& searchPaths, char separator = PathName::PathNameDelimiter)
+string Concat(const initializer_list<string>& searchPaths, char separator = PathNameUtil::PathNameDelimiter)
 {
   string result;
   for (const string& s : searchPaths)
@@ -2107,9 +2110,9 @@ void IniTeXMFApp::RegisterOtherRoots()
   for (const OtherTeX& other : otherTeXDists)
   {
     const string& roots = (session->IsAdminMode() ? other.startupConfig.commonRoots : other.startupConfig.userRoots);
-    for (const string& r : StringUtil::Split(roots, PathName::PathNameDelimiter))
+    for (const string& r : StringUtil::Split(roots, PathNameUtil::PathNameDelimiter))
     {
-      otherRoots.push_back(r);
+      otherRoots.push_back(PathName(r));
     }
   }
   if (otherRoots.empty())
@@ -2403,7 +2406,7 @@ void IniTeXMFApp::Run(int argc, const char* argv[])
 
     case OPT_REGISTER_ROOT:
 
-      registerRoots.push_back(optArg);
+      registerRoots.push_back(PathName(optArg));
       break;
 
     case OPT_REMOVE_FILE:
@@ -2457,7 +2460,7 @@ void IniTeXMFApp::Run(int argc, const char* argv[])
 
     case OPT_UNREGISTER_ROOT:
 
-      unregisterRoots.push_back(optArg);
+      unregisterRoots.push_back(PathName(optArg));
       break;
 
     case OPT_UNREGISTER_SHELL_FILE_TYPES:
@@ -2503,9 +2506,9 @@ void IniTeXMFApp::Run(int argc, const char* argv[])
   if (optVersion)
   {
     cout
-      << Utils::MakeProgramVersionString(TheNameOfTheGame, MIKTEX_COMPONENT_VERSION_STR) << endl
+      << Utils::MakeProgramVersionString(TheNameOfTheGame, VersionNumber(MIKTEX_COMPONENT_VERSION_STR)) << endl
       << endl
-      << "Copyright (C) 1996-2019 Christian Schenk" << endl
+      << "Copyright (C) 1996-2020 Christian Schenk" << endl
       << "This is free software; see the source for copying conditions.  There is NO" << endl
       << "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE." << endl;
     return;
@@ -2513,13 +2516,13 @@ void IniTeXMFApp::Run(int argc, const char* argv[])
 
   if (!logFile.empty())
   {
-    auto mode = File::Exists(logFile) ? ios_base::app : ios_base::out;
-    logStream = File::CreateOutputStream(logFile, mode);
+    auto mode = File::Exists(PathName(logFile)) ? ios_base::app : ios_base::out;
+    logStream = File::CreateOutputStream(PathName(logFile), mode);
   }
 
   if (optPortable)
   {
-    CreatePortableSetup(portableRoot);
+    CreatePortableSetup(PathName(portableRoot));
   }
 
   if (!startupConfig.userRoots.empty()
@@ -2600,9 +2603,9 @@ void IniTeXMFApp::Run(int argc, const char* argv[])
     PrintOnly(fmt::format("fndbadd {}", Q_(fileName)));
     if (!printOnly)
     {
-      if (!Fndb::FileExists(fileName))
+      if (!Fndb::FileExists(PathName(fileName)))
       {
-        records.push_back({ fileName });
+        records.push_back({ PathName(fileName) });
       }
       else
       {
@@ -2622,9 +2625,9 @@ void IniTeXMFApp::Run(int argc, const char* argv[])
     PrintOnly(fmt::format("fndbremove {}", Q_(fileName)));
     if (!printOnly)
     {
-      if (Fndb::FileExists(fileName))
+      if (Fndb::FileExists(PathName(fileName)))
       {
-        paths.push_back(fileName);
+        paths.push_back(PathName(fileName));
       }
       else
       {
@@ -2706,7 +2709,7 @@ void IniTeXMFApp::Run(int argc, const char* argv[])
     {
       for (const string& r : updateRoots)
       {
-        UpdateFilenameDatabase(r);
+        UpdateFilenameDatabase(PathName(r));
       }
     }
   }
