@@ -52,7 +52,9 @@ enum class Engine
   pdfTeX,
   XeTeX,
   LuaHBTeX,
+#if defined(WITH_OMEGA)
   Omega,
+#endif
 };
 
 class PdfConfigValues :
@@ -133,10 +135,12 @@ private:
     {
       this->engine = Engine::LuaHBTeX;
     }
+#if defined(WITH_OMEGA)
     else if (Utils::EqualsIgnoreCase(engine, "omega"))
     {
       this->engine = Engine::Omega;
     }
+#endif
     else
     {
       FatalError(fmt::format(T_("Unknown engine: {0}"), engine));
@@ -164,8 +168,10 @@ public:
       return "xetex";
     case Engine::LuaHBTeX:
       return "luahbtex";
+#if defined(WITH_OMEGA)
     case Engine::Omega:
       return "omega";
+#endif
     }
     MIKTEX_UNEXPECTED();
   }
@@ -185,8 +191,10 @@ private:
       return MIKTEX_XETEX_EXE;
     case Engine::LuaHBTeX:
       return MIKTEX_LUAHBTEX_EXE;
+#if defined(WITH_OMEGA)
     case Engine::Omega:
       return MIKTEX_OMEGA_EXE;
+#endif
     }
     MIKTEX_UNEXPECTED();
   }
@@ -374,7 +382,7 @@ void MakeFmt::ParsePdfConfigFile(const PathName& cfgFile, PdfConfigValues& value
 void MakeFmt::InstallPdftexConfigTeX() const
 {
   PdfConfigValues pdfConfigValues = ParsePdfConfigFiles();
-  session->ConfigureFile(MIKTEX_PATH_PDFTEXCONFIG_TEX, &pdfConfigValues);
+  session->ConfigureFile(PathName(MIKTEX_PATH_PDFTEXCONFIG_TEX), &pdfConfigValues);
 }
 
 void MakeFmt::Run(int argc, const char** argv)
@@ -395,7 +403,7 @@ void MakeFmt::Run(int argc, const char** argv)
   }
 
   // pretend to be the engine / format
-  if (!(destinationName == GetEngineName()))
+  if (!(destinationName == PathName(GetEngineName())))
   {
     session->PushAppName(GetEngineName());
   }
@@ -403,7 +411,7 @@ void MakeFmt::Run(int argc, const char** argv)
 
   // find the TeX input file
   PathName inputFile;
-  FindInputFile(name, inputFile);
+  FindInputFile(PathName(name), inputFile);
 
   // create destination directory
   CreateDestinationDirectory();
@@ -426,7 +434,7 @@ void MakeFmt::Run(int argc, const char** argv)
   arguments.push_back("--initialize");
   arguments.push_back("--interaction="s + "nonstopmode");
   arguments.push_back("--halt-on-error");
-  if (destinationName != GetEngineName())
+  if (destinationName != PathName(GetEngineName()))
   {
     arguments.push_back("--alias=" + destinationName.ToString());
   }
