@@ -199,7 +199,7 @@ The return value will be the directory of the executable, e.g.: \.{c:/TeX/bin}
 
 static char *ex_selfdir(char *argv0)
 {
-#if defined(WIN32) && !defined(MIKTEX)
+#if !defined(MIKTEX) && defined(WIN32)
 #if defined(__MINGW32__)
     char path[PATH_MAX], *fp;
     /*tex SearchPath() always gives back an absolute directory */
@@ -383,10 +383,12 @@ static int recorderoption = 0;
 
 static void parse_options(int ac, char **av)
 {
+#if !defined(MIKTEX)
 #ifdef WIN32
     /*tex We save |argc| and |argv|. */
     int sargc = argc;
     char **sargv = argv;
+#endif
 #endif
      /*tex The `getopt' return code. */
     int g;
@@ -500,6 +502,18 @@ static void parse_options(int ac, char **av)
                 strncpy(output_comment, optarg, 255);
                 output_comment[255] = 0;
             }
+#if defined(MIKTEX)
+        } else if (ARGUMENT_IS("enable-write18") || ARGUMENT_IS("shell-escape")) {
+          shellenabledp = 1;
+          if (miktex_allow_unrestricted_shell_escape())
+          {
+            restrictedshell = 0;
+          }
+          else
+          {
+            restrictedshell = 1;
+          }
+#endif
 #if defined(MIKTEX)
         } else if (ARGUMENT_IS("restrict-write18") || ARGUMENT_IS("shell-restricted")) {
 #else
@@ -607,7 +621,14 @@ static void parse_options(int ac, char **av)
         }
     } else if (argv[optind] && argv[optind][0] == '&') {
         dump_name = xstrdup(argv[optind] + 1);
+#if defined(MIKTEX_WINDOWS)
+    } else if (argv[optind] && (argv[optind][0] != '\\' || miktex_is_fully_qualified_path(argv[optind]))) {
+#else
     } else if (argv[optind] && argv[optind][0] != '\\') {
+#endif
+#if defined(MIKTEX_WINDOWS)
+      miktex_convert_to_unix(argv[optind]);
+#endif
         if (argv[optind][0] == '*') {
             input_name = xstrdup(argv[optind] + 1);
         } else {
@@ -630,6 +651,7 @@ static void parse_options(int ac, char **av)
                 input_name = firstfile;
             }
         }
+#if !defined(MIKTEX)
 #ifdef WIN32
     } else if (sargc > 1 && sargv[sargc-1] && sargv[sargc-1][0] != '-' &&
                sargv[sargc-1][0] != '\\') {
@@ -645,6 +667,7 @@ static void parse_options(int ac, char **av)
         if (safer_option)      /* --safer implies --nosocket */
             nosocket_option = 1;
         return;
+#endif
 #endif
     }
     /*tex |--safer| implies |--nosocket| */
@@ -713,19 +736,11 @@ static void init_kpse(void)
                 user_progname = remove_suffix (input_name);
             }
             if (!user_progname) {
-#if defined(MIKTEX)
-                user_progname = miktex_program_basename(argv[0]);
-#else
                 user_progname = kpse_program_basename(argv[0]);
-#endif
             }
         } else {
             if (!dump_name) {
-#if defined(MIKTEX)
-                dump_name = miktex_program_basename(argv[0]);
-#else
                 dump_name = kpse_program_basename(argv[0]);
-#endif
             }
             user_progname = dump_name;
         }
@@ -1060,11 +1075,7 @@ void lua_initialize(int ac, char **av)
     banner = xmalloc(len);
     sprintf(banner, fmt, luatex_version_string);
     luatex_banner = banner;
-#if defined(MIKTEX)
-    kpse_invocation_name = miktex_program_basename(argv[0]);
-#else
     kpse_invocation_name = kpse_program_basename(argv[0]);
-#endif
     /*tex be `luac' */
     if (argc >1) {
 #ifdef LuajitTeX
