@@ -2744,7 +2744,7 @@ boolean open_dvi_output(C4P::FileRoot& dviFile)
     processStartInfo.RedirectStandardInput = true;
     dvipdfmxProcess = MiKTeX::Core::Process::Start(processStartInfo);
     dviFile.Attach(dvipdfmxProcess->get_StandardInput(), true);
-    MiKTeX::TeXAndFriends::WebAppInputLine::GetWebAppInputLine()->SetNameOfFile(outPath.GetData());
+    MiKTeX::TeXAndFriends::WebAppInputLine::GetWebAppInputLine()->SetNameOfFile(outPath);
     return 1;
   }
 }
